# Copyright 2014 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.api.network import base
from tempest.common import utils
from tempest import config
from tempest.lib.common.utils import test_utils
from tempest.lib import decorators

CONF = config.CONF


class FloatingIPAdminTestJSON(base.BaseAdminNetworkTest):
    """Test floating ips"""

    credentials = ['primary', 'alt', 'admin']

    @classmethod
    def skip_checks(cls):
        super(FloatingIPAdminTestJSON, cls).skip_checks()
        if not utils.is_extension_enabled('router', 'network'):
            msg = "router extension not enabled."
            raise cls.skipException(msg)
        if not CONF.network.public_network_id:
            msg = "The public_network_id option must be specified."
            raise cls.skipException(msg)
        if not CONF.network_feature_enabled.floating_ips:
            raise cls.skipException("Floating ips are not available")

    @classmethod
    def setup_clients(cls):
        super(FloatingIPAdminTestJSON, cls).setup_clients()
        cls.alt_floating_ips_client = cls.os_alt.floating_ips_client

    @classmethod
    def resource_setup(cls):
        super(FloatingIPAdminTestJSON, cls).resource_setup()
        cls.ext_net_id = CONF.network.public_network_id
        cls.floating_ip = cls.create_floatingip(cls.ext_net_id)
        cls.network = cls.create_network()
        subnet = cls.create_subnet(cls.network)
        router = cls.create_router(external_network_id=cls.ext_net_id)
        cls.create_router_interface(router['id'], subnet['id'])
        cls.port = cls.create_port(cls.network)

    @decorators.idempotent_id('64f2100b-5471-4ded-b46c-ddeeeb4f231b')
    def test_list_floating_ips_from_admin_and_nonadmin(self):
        """Test listing floating ips from admin and non admin users

        This test performs below operations:
        1. Create couple floating ips for admin and non-admin users.
        2. Verify if admin can access all floating ips including other user
        and non-admin user can only access its own floating ips.
        """
        # Create floating ip from admin user
        floating_ip_admin = self.admin_floating_ips_client.create_floatingip(
            floating_network_id=self.ext_net_id)
        self.addCleanup(
            test_utils.call_and_ignore_notfound_exc,
            self.admin_floating_ips_client.delete_floatingip,
            floating_ip_admin['floatingip']['id'])
        # Create floating ip from alt user
        body = self.alt_floating_ips_client.create_floatingip(
            floating_network_id=self.ext_net_id)
        floating_ip_alt = body['floatingip']
        self.addCleanup(
            test_utils.call_and_ignore_notfound_exc,
            self.alt_floating_ips_client.delete_floatingip,
            floating_ip_alt['id'])
        # List floating ips from admin
        body = self.admin_floating_ips_client.list_floatingips()
        floating_ip_ids_admin = [f['id'] for f in body['floatingips']]
        # Check that admin sees all floating ips
        self.assertIn(self.floating_ip['id'], floating_ip_ids_admin)
        self.assertIn(floating_ip_admin['floatingip']['id'],
                      floating_ip_ids_admin)
        self.assertIn(floating_ip_alt['id'], floating_ip_ids_admin)
        # List floating ips from nonadmin
        body = self.floating_ips_client.list_floatingips()
        floating_ip_ids = [f['id'] for f in body['floatingips']]
        # Check that nonadmin user doesn't see floating ip created from admin
        # and floating ip that is created in another project (alt user)
        self.assertIn(self.floating_ip['id'], floating_ip_ids)
        self.assertNotIn(floating_ip_admin['floatingip']['id'],
                         floating_ip_ids)
        self.assertNotIn(floating_ip_alt['id'], floating_ip_ids)

    @decorators.idempotent_id('32727cc3-abe2-4485-a16e-48f2d54c14f2')
    def test_create_list_show_floating_ip_with_tenant_id_by_admin(self):
        """Verify if admin can create/list/show floating ip with tenant id"""
        # Creates a floating IP
        body = self.admin_floating_ips_client.create_floatingip(
            floating_network_id=self.ext_net_id,
            project_id=self.network['project_id'],
            port_id=self.port['id'])
        created_floating_ip = body['floatingip']
        self.addCleanup(
            test_utils.call_and_ignore_notfound_exc,
            self.floating_ips_client.delete_floatingip,
            created_floating_ip['id'])
        self.assertIsNotNone(created_floating_ip['id'])
        self.assertIsNotNone(created_floating_ip['project_id'])
        self.assertIsNotNone(created_floating_ip['floating_ip_address'])
        self.assertEqual(created_floating_ip['port_id'], self.port['id'])
        self.assertEqual(created_floating_ip['floating_network_id'],
                         self.ext_net_id)
        port = self.port['fixed_ips']
        self.assertEqual(created_floating_ip['fixed_ip_address'],
                         port[0]['ip_address'])
        # Verifies the details of a floating_ip
        floating_ip = self.admin_floating_ips_client.show_floatingip(
            created_floating_ip['id'])
        shown_floating_ip = floating_ip['floatingip']
        self.assertEqual(shown_floating_ip['id'], created_floating_ip['id'])
        self.assertEqual(shown_floating_ip['floating_network_id'],
                         self.ext_net_id)
        self.assertEqual(shown_floating_ip['project_id'],
                         self.network['project_id'])
        self.assertEqual(shown_floating_ip['floating_ip_address'],
                         created_floating_ip['floating_ip_address'])
        self.assertEqual(shown_floating_ip['port_id'], self.port['id'])
        # Verify the floating ip exists in the list of all floating_ips
        floating_ips = self.admin_floating_ips_client.list_floatingips()
        floatingip_id_list = [f['id'] for f in floating_ips['floatingips']]
        self.assertIn(created_floating_ip['id'], floatingip_id_list)



# Copyright 2013 IBM Corporation.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from urllib import parse as urllib

from oslo_serialization import jsonutils as json

from tempest.lib.api_schema.response.compute.v2_1 \
    import hypervisors as schemav21
from tempest.lib.api_schema.response.compute.v2_28 \
    import hypervisors as schemav228
from tempest.lib.api_schema.response.compute.v2_33 \
    import hypervisors as schemav233
from tempest.lib.api_schema.response.compute.v2_53 \
    import hypervisors as schemav253
from tempest.lib.common import rest_client
from tempest.lib.services.compute import base_compute_client


class HypervisorClient(base_compute_client.BaseComputeClient):

    schema_versions_info = [
        {'min': None, 'max': '2.27', 'schema': schemav21},
        {'min': '2.28', 'max': '2.32', 'schema': schemav228},
        {'min': '2.33', 'max': '2.52', 'schema': schemav233},
        {'min': '2.53', 'max': None, 'schema': schemav253}]

    def list_hypervisors(self, detail=False, **kwargs):
        """List hypervisors information."""
        url = 'os-hypervisors'
        schema = self.get_schema(self.schema_versions_info)
        _schema = schema.list_search_hypervisors
        if detail:
            url += '/detail'
            _schema = schema.list_hypervisors_detail
        if kwargs:
            url += '?%s' % urllib.urlencode(kwargs)

        resp, body = self.get(url)
        body = json.loads(body)
        self.validate_response(_schema, resp, body)
        return rest_client.ResponseBody(resp, body)

    def show_hypervisor(self, hypervisor_id, **kwargs):
        """Display the details of the specified hypervisor."""
        url = 'os-hypervisors/%s' % hypervisor_id
        if kwargs:
            url += '?%s' % urllib.urlencode(kwargs)
        resp, body = self.get('os-hypervisors/%s' % hypervisor_id)
        body = json.loads(body)
        schema = self.get_schema(self.schema_versions_info)
        self.validate_response(schema.get_hypervisor, resp, body)
        return rest_client.ResponseBody(resp, body)

    def list_servers_on_hypervisor(self, hypervisor_name):
        """List instances belonging to the specified hypervisor."""
        resp, body = self.get('os-hypervisors/%s/servers' % hypervisor_name)
        body = json.loads(body)
        schema = self.get_schema(self.schema_versions_info)
        self.validate_response(schema.get_hypervisors_servers, resp, body)
        return rest_client.ResponseBody(resp, body)

    def show_hypervisor_statistics(self):
        """Get hypervisor statistics over all compute nodes."""
        resp, body = self.get('os-hypervisors/statistics')
        body = json.loads(body)
        schema = self.get_schema(self.schema_versions_info)
        self.validate_response(schema.get_hypervisor_statistics, resp, body)
        return rest_client.ResponseBody(resp, body)

    def show_hypervisor_uptime(self, hypervisor_id):
        """Display the uptime of the specified hypervisor."""
        resp, body = self.get('os-hypervisors/%s/uptime' % hypervisor_id)
        body = json.loads(body)
        schema = self.get_schema(self.schema_versions_info)
        self.validate_response(schema.get_hypervisor_uptime, resp, body)
        return rest_client.ResponseBody(resp, body)

    def search_hypervisor(self, hypervisor_name):  # noqa
        # NOTE: This noqa is for passing T110 check and we cannot rename
        #       to keep backwards compatibility.
        """Search specified hypervisor."""
        resp, body = self.get('os-hypervisors/%s/search' % hypervisor_name)
        body = json.loads(body)
        schema = self.get_schema(self.schema_versions_info)
        self.validate_response(schema.list_search_hypervisors, resp, body)
        return rest_client.ResponseBody(resp, body)



# Copyright 2018 AT&T Corporation.
# All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import fixtures

from oslo_serialization import jsonutils as json
from tempest import clients
from tempest.cmd import cleanup_service
from tempest import config
from tempest.lib import exceptions
from tempest.tests import base
from tempest.tests import fake_config
from tempest.tests.lib import fake_credentials
from tempest.tests.lib import fake_http


class TestBaseService(base.TestCase):

    class TestException(cleanup_service.BaseService):
        def delete(self):
            raise exceptions.NotImplemented

        def dry_run(self):
            raise exceptions.NotImplemented

        def save_state(self):
            raise exceptions.NotImplemented

    def test_base_service_init(self):
        kwargs = {'data': {'data': 'test'},
                  'is_dry_run': False,
                  'resource_list_json': {'resp': 'data'},
                  'saved_state_json': {'saved': 'data'},
                  'is_preserve': False,
                  'is_resource_list': False,
                  'is_save_state': True,
                  'prefix': 'tempest',
                  'tenant_id': 'project_id',
                  'got_exceptions': []}
        base = cleanup_service.BaseService(kwargs)
        self.assertEqual(base.data, kwargs['data'])
        self.assertFalse(base.is_dry_run)
        self.assertEqual(base.resource_list_json, kwargs['resource_list_json'])
        self.assertEqual(base.saved_state_json, kwargs['saved_state_json'])
        self.assertFalse(base.is_preserve)
        self.assertFalse(base.is_resource_list)
        self.assertTrue(base.is_save_state)
        self.assertEqual(base.tenant_filter['project_id'], kwargs['tenant_id'])
        self.assertEqual(base.got_exceptions, kwargs['got_exceptions'])
        self.assertEqual(base.prefix, kwargs['prefix'])

    def test_not_implemented_ex(self):
        kwargs = {'data': {'data': 'test'},
                  'is_dry_run': False,
                  'resource_list_json': {'resp': 'data'},
                  'saved_state_json': {'saved': 'data'},
                  'is_preserve': False,
                  'is_resource_list': False,
                  'is_save_state': False,
                  'prefix': 'tempest',
                  'tenant_id': 'project_id',
                  'got_exceptions': []}
        base = self.TestException(kwargs)
        # delete
        base.run()
        self.assertEqual(len(base.got_exceptions), 1)
        # save_state
        base.save_state = True
        base.run()
        self.assertEqual(len(base.got_exceptions), 2)
        # dry_run
        base.is_dry_run = True
        base.run()
        self.assertEqual(len(base.got_exceptions), 3)


class MockFunctionsBase(base.TestCase):

    def _create_response(self, body, status, headers):
        if status:
            if body:
                body = json.dumps(body)
            resp = fake_http.fake_http_response(headers, status=status), body
            return resp
        else:
            return body

    def _create_fixtures(self, fixtures_to_make):
        mocked_fixtures = []
        for fixture in fixtures_to_make:
            func, body, status = fixture
            mocked_response = self._create_response(body, status, None)
            if mocked_response == 'error':
                mocked_func = self.useFixture(fixtures.MockPatch(
                    func, side_effect=Exception("error")))
            else:
                mocked_func = self.useFixture(fixtures.MockPatch(
                    func, return_value=mocked_response))
            mocked_fixtures.append(mocked_func)
        return mocked_fixtures

    def run_function_with_mocks(self, function_to_run, functions_to_mock):
        """Mock a service client function for testing.

        :param function_to_run: The service client function to call.
        :param functions_to_mock: a list of tuples containing the function
               to mock, the response body, and the response status.
               EX:
               ('tempest.lib.common.rest_client.RestClient.get',
                {'users': ['']},
                200)
        """
        mocked_fixtures = self._create_fixtures(functions_to_mock)
        func_return = function_to_run()
        return func_return, mocked_fixtures


class BaseCmdServiceTests(MockFunctionsBase):

    def setUp(self):
        super(BaseCmdServiceTests, self).setUp()
        self.useFixture(fake_config.ConfigFixture())
        self.patchobject(config, 'TempestConfigPrivate',
                         fake_config.FakePrivate)
        self.useFixture(fixtures.MockPatch(
            'tempest.cmd.cleanup_service._get_network_id',
            return_value=''))
        cleanup_service.init_conf()
        self.conf_values = {"flavors": cleanup_service.CONF_FLAVORS[0],
                            "images": cleanup_service.CONF_IMAGES[0],
                            "projects": cleanup_service.CONF_PROJECTS[0],
                            "users": cleanup_service.CONF_USERS[0],
                            "networks": cleanup_service.CONF_PUB_NETWORK,
                            "security_groups":
                                cleanup_service.CONF_PROJECTS[0],
                            "ports": cleanup_service.CONF_PUB_NETWORK,
                            "routers": cleanup_service.CONF_PUB_ROUTER,
                            "subnetpools": cleanup_service.CONF_PROJECTS[0],
                            }

    saved_state = {
        # Static list to ensure global service saved items are not deleted
        "users": {'32rwef64245tgr20121qw324bgg': 'Lightning'},
        "flavors": {'42': 'm1.tiny'},
        "images": {'34yhwr-4t3q': 'stratus-0.3.2-x86_64-disk'},
        "roles": {'3efrt74r45hn': 'president'},
        "projects": {'f38ohgp93jj032': 'manhattan'},
        "domains": {'default': 'Default'},
        # Static list to ensure project service saved items are not deleted
        "snapshots": {'1ad4c789-7e8w-4dwg-afc5': 'saved-snapshot'},
        "servers": {'7a6d4v7w-36ds-4216': 'saved-server'},
        "server_groups": {'as6d5f7g-46ca-475e': 'saved-server-group'},
        "keypairs": {'saved-key-pair': {
            'fingerprint': '7e:eb:ab:24',
            'name': 'saved-key-pair'
        }},
        "volumes": {'aa77asdf-1234': 'saved-volume'},
        "networks": {'6722fc13-4319': {
            'id': '6722fc13-4319',
            'name': 'saved-network'
        }},
        "floatingips": {'9e82d248-408a': {
            'id': '9e82d248-408a',
            'status': 'ACTIVE'
        }},
        "routers": {'4s5w34hj-id44': 'saved-router'},
        "metering_label_rules": {'93a973ce-4dc5': {
            'direction': 'ingress',
            'id': '93a973ce-4dc5'
        }},
        "metering_labels": {'723b346ce866-4c7q': 'saved-label'},
        "ports": {'aa74aa4v-741a': 'saved-port'},
        "security_groups": {'7q844add-3697': 'saved-sec-group'},
        "subnets": {'55ttda4a-2584': 'saved-subnet'},
        "subnetpools": {'8acf64c1-43fc': 'saved-subnet-pool'},
        "regions": {'RegionOne': {}}
    }

    resource_list = {
        "keypairs": {'saved-key-pair': ""}
    }

    # Mocked methods
    get_method = 'tempest.lib.common.rest_client.RestClient.get'
    delete_method = 'tempest.lib.common.rest_client.RestClient.delete'
    log_method = 'tempest.cmd.cleanup_service.LOG.exception'
    filter_saved_state = 'tempest.cmd.cleanup_service.' \
                         'BaseService._filter_out_ids_from_saved'
    filter_resource_list = 'tempest.cmd.cleanup_service.' \
                           'BaseService._filter_by_resource_list'
    filter_prefix = 'tempest.cmd.cleanup_service.BaseService._filter_by_prefix'
    # Override parameters
    service_class = 'BaseService'
    response = None
    service_name = 'default'

    def _create_cmd_service(self, service_type, is_save_state=False,
                            is_preserve=False, is_dry_run=False,
                            prefix='', is_resource_list=False):
        creds = fake_credentials.FakeKeystoneV3Credentials()
        os = clients.Manager(creds)
        return getattr(cleanup_service, service_type)(
            os,
            is_resource_list=is_resource_list,
            is_save_state=is_save_state,
            is_preserve=is_preserve,
            is_dry_run=is_dry_run,
            prefix=prefix,
            project_id='b8e3ece07bb049138d224436756e3b57',
            data={},
            resource_list_json=self.resource_list,
            saved_state_json=self.saved_state
            )

    def _test_delete(self, mocked_fixture_tuple_list, fail=False):
        serv = self._create_cmd_service(self.service_class)
        resp, fixtures = self.run_function_with_mocks(
            serv.run,
            mocked_fixture_tuple_list,
        )
        for fixture in fixtures:
            if fixture.mock.return_value == 'validate':
                fixture.mock.assert_called()
            elif fail is False and fixture.mock.return_value == 'exception':
                fixture.mock.assert_not_called()
            elif self.service_name in self.saved_state.keys():
                fixture.mock.assert_called_once()
                for key in self.saved_state[self.service_name].keys():
                    self.assertNotIn(key, fixture.mock.call_args[0][0])
            else:
                fixture.mock.assert_called_once()
        self.assertFalse(serv.data)

    def _test_dry_run_true(self, mocked_fixture_tuple_list):
        serv = self._create_cmd_service(self.service_class, is_dry_run=True)
        _, fixtures = self.run_function_with_mocks(
            serv.run,
            mocked_fixture_tuple_list
        )
        for fixture in fixtures:
            if fixture.mock.return_value == 'delete':
                fixture.mock.assert_not_called()
            elif self.service_name in self.saved_state.keys():
                fixture.mock.assert_called_once()
                for key in self.saved_state[self.service_name].keys():
                    self.assertNotIn(key, fixture.mock.call_args[0][0])
            else:
                fixture.mock.assert_called_once()

    def _test_saved_state_true(self, mocked_fixture_tuple_list):
        serv = self._create_cmd_service(self.service_class, is_save_state=True)
        _, fixtures = self.run_function_with_mocks(
            serv.run,
            mocked_fixture_tuple_list
        )
        for item in self.response[self.service_name]:
            self.assertIn(item['id'],
                          serv.data[self.service_name])
        for fixture in fixtures:
            fixture.mock.assert_called_once()

    def _test_is_preserve_true(self, mocked_fixture_tuple_list):
        serv = self._create_cmd_service(self.service_class, is_preserve=True)
        resp, fixtures = self.run_function_with_mocks(
            serv.list,
            mocked_fixture_tuple_list
        )
        for fixture in fixtures:
            fixture.mock.assert_called_once()
        self.assertIn(resp[0], self.response[self.service_name])
        for rsp in resp:
            self.assertNotIn(rsp['id'], self.conf_values.values())
            self.assertNotIn(rsp['name'], self.conf_values.values())

    def _test_prefix_opt_precedence(self, delete_mock):
        serv = self._create_cmd_service(
            self.service_class, is_resource_list=True, prefix='tempest')
        _, fixtures = self.run_function_with_mocks(
            serv.run,
            delete_mock
        )

        # Check that prefix was used for filtering
        fixtures[2].mock.assert_called_once()

        # Check that neither saved_state.json nor resource list was
        # used for filtering
        fixtures[0].mock.assert_not_called()
        fixtures[1].mock.assert_not_called()

    def _test_resource_list_opt_precedence(self, delete_mock):
        serv = self._create_cmd_service(
            self.service_class, is_resource_list=True)
        _, fixtures = self.run_function_with_mocks(
            serv.run,
            delete_mock
        )

        # Check that resource list was used for filtering
        fixtures[1].mock.assert_called_once()

        # Check that neither saved_state.json nor prefix was
        # used for filtering
        fixtures[0].mock.assert_not_called()
        fixtures[2].mock.assert_not_called()


class TestSnapshotService(BaseCmdServiceTests):

    service_class = 'SnapshotService'
    service_name = 'snapshots'
    response = {
        "snapshots": [
            {
                "status": "available",
                "metadata": {
                    "name": "test"
                },
                "name": "test-volume-snapshot",
                "volume_id": "173f7b48-c4c1-4e70-9acc-086b39073506",
                "created_at": "2015-11-29T02:25:51.000000",
                "size": 1,
                "updated_at": "2015-11-20T05:36:40.000000",
                "id": "b1323cda-8e4b-41c1-afc5-2fc791809c8c",
                "description": "volume snapshot"
            },
            {
                "status": "available",
                "name": "saved-snapshot",
                "metadata": {},
                "id": "1ad4c789-7e8w-4dwg-afc5",
                "size": 1,
                "volume_id": "af7c41be-1ff6-4233-a690-7ed61c34347f",
                "created_at": "2015-11-20T05:39:40.000000",
                "updated_at": "2015-11-20T05:39:40.000000",
                "description": "snapshot in saved state"
            }
        ]
    }

    def test_delete_fail(self):
        delete_mock = [(self.get_method, self.response, 200),
                       (self.delete_method, 'error', None),
                       (self.log_method, 'exception', None)]
        self._test_delete(delete_mock, fail=True)

    def test_delete_pass(self):
        delete_mock = [(self.get_method, self.response, 200),
                       (self.delete_method, None, 202),
                       (self.log_method, 'exception', None)]
        self._test_delete(delete_mock)

    def test_dry_run(self):
        dry_mock = [(self.get_method, self.response, 200),
                    (self.delete_method, "delete", None)]
        self._test_dry_run_true(dry_mock)

    def test_save_state(self):
        self._test_saved_state_true([(self.get_method, self.response, 200)])

    def test_prefix_opt_precedence(self):
        delete_mock = [(self.filter_saved_state, [], None),
                       (self.filter_resource_list, [], None),
                       (self.filter_prefix, [], None),
                       (self.get_method, self.response, 200),
                       (self.delete_method, 'error', None),
                       (self.log_method, 'exception', None)]
        self._test_prefix_opt_precedence(delete_mock)

    def test_resource_list_opt_precedence(self):
        delete_mock = [(self.filter_saved_state, [], None),
                       (self.filter_resource_list, [], None),
                       (self.filter_prefix, [], None),
                       (self.get_method, self.response, 200),
                       (self.delete_method, 'error', None),
                       (self.log_method, 'exception', None)]
        self._test_resource_list_opt_precedence(delete_mock)


class TestServerService(BaseCmdServiceTests):

    service_class = 'ServerService'
    service_name = 'servers'
    response = {
        "servers": [
            {
                "id": "22c91117-08de-4894-9aa9-6ef382400985",
                "links": [
                    {
                        "href": "http://openstack.example.com/v2/6f70-6ef0985",
                        "rel": "self"
                    },
                    {
                        "href": "http://openstack.example.com/6f70656e7-6ef35",
                        "rel": "bookmark"
                    }
                ],
                "name": "new-server-test"
            },
            {
                "id": "7a6d4v7w-36ds-4216",
                "links": [
                    {
                        "href": "http://openstack.example.com/v2/6f70-6ef0985",
                        "rel": "self"
                    },
                    {
                        "href": "http://openstack.example.com/6f70656e7-6ef35",
                        "rel": "bookmark"
                    }
                ],
                "name": "saved-server"
            }
        ]
    }

    def test_delete_fail(self):
        delete_mock = [(self.get_method, self.response, 200),
                       (self.delete_method, 'error', None),
                       (self.log_method, 'exception', None)]
        self._test_delete(delete_mock, fail=True)

    def test_delete_pass(self):
        delete_mock = [(self.get_method, self.response, 200),
                       (self.delete_method, None, 204),
                       (self.log_method, 'exception', None)]
        self._test_delete(delete_mock)

    def test_dry_run(self):
        dry_mock = [(self.get_method, self.response, 200),
                    (self.delete_method, "delete", None)]
        self._test_dry_run_true(dry_mock)

    def test_save_state(self):
        self._test_saved_state_true([(self.get_method, self.response, 200)])

    def test_prefix_opt_precedence(self):
        delete_mock = [(self.filter_saved_state, [], None),
                       (self.filter_resource_list, [], None),
                       (self.filter_prefix, [], None),
                       (self.get_method, self.response, 200),
                       (self.delete_method, 'error', None),
                       (self.log_method, 'exception', None)]
        self._test_prefix_opt_precedence(delete_mock)

    def test_resource_list_opt_precedence(self):
        delete_mock = [(self.filter_saved_state, [], None),
                       (self.filter_resource_list, [], None),
                       (self.filter_prefix, [], None),
                       (self.get_method, self.response, 200),
                       (self.delete_method, 'error', None),
                       (self.log_method, 'exception', None)]
        self._test_resource_list_opt_precedence(delete_mock)


class TestServerGroupService(BaseCmdServiceTests):

    service_class = 'ServerGroupService'
    service_name = 'server_groups'
    validate_response = ('tempest.lib.services.compute.server_groups_client'
                         '.ServerGroupsClient.validate_response')

    response = {
        "server_groups": [
            {
                "id": "616fb98f-46ca-475e-917e-2563e5a8cd19",
                "name": "test",
                "policy": "anti-affinity",
                "rules": {"max_server_per_host": 3},
                "members": [],
                "project_id": "6f70656e737461636b20342065766572",
                "user_id": "fake"
            },
            {
                "id": "as6d5f7g-46ca-475e",
                "name": "saved-server-group"
            }
        ]
    }

    def test_delete_fail(self):
        delete_mock = [(self.get_method, self.response, 200),
                       (self.validate_response, 'validate', None),
                       (self.delete_method, 'error', None),
                       (self.log_method, 'exception', None)]
        self._test_delete(delete_mock, fail=True)

    def test_delete_pass(self):
        delete_mock = [(self.get_method, self.response, 200),
                       (self.validate_response, 'validate', None),
                       (self.delete_method, None, 204),
                       (self.log_method, 'exception', None)]
        self._test_delete(delete_mock)

    def test_dry_run(self):
        dry_mock = [(self.get_method, self.response, 200),
                    (self.validate_response, 'validate', None),
                    (self.delete_method, "delete", None)]
        self._test_dry_run_true(dry_mock)

    def test_save_state(self):
        self._test_saved_state_true([(self.get_method, self.response, 200),
                                     (self.validate_response, 'validate', None)
                                     ])

    def test_prefix_opt_precedence(self):
        delete_mock = [(self.filter_saved_state, [], None),
                       (self.filter_resource_list, [], None),
                       (self.filter_prefix, [], None),
                       (self.get_method, self.response, 200),
                       (self.validate_response, 'validate', None),
                       (self.delete_method, 'error', None),
                       (self.log_method, 'exception', None)]
        self._test_prefix_opt_precedence(delete_mock)

    def test_resource_list_opt_precedence(self):
        delete_mock = [(self.filter_saved_state, [], None),
                       (self.filter_resource_list, [], None),
                       (self.filter_prefix, [], None),
                       (self.get_method, self.response, 200),
                       (self.validate_response, 'validate', None),
                       (self.delete_method, 'error', None),
                       (self.log_method, 'exception', None)]
        self._test_resource_list_opt_precedence(delete_mock)


class TestKeyPairService(BaseCmdServiceTests):

    service_class = 'KeyPairService'
    service_name = 'keypairs'
    validate_response = ('tempest.lib.services.compute.keypairs_client'
                         '.KeyPairsClient.validate_response')
    response = {
        "keypairs": [
            {
                "keypair": {
                    "fingerprint": "7e:eb:ab:24:ba:d1:e1:88:ae:9a:fb:66:53:bd",
                    "name": "keypair-5d935425-31d5-48a7-a0f1-e76e9813f2c3",
                    "type": "ssh",
                    "public_key": "ssh-rsa AAAAB3NzaC1yc2EAAAADAQABAAABAQCkF\n"
                }
            },
            {
                "keypair": {
                    "fingerprint": "7e:eb:ab:24",
                    "name": "saved-key-pair"
                }
            }
        ]
    }

    def _test_saved_state_true(self, mocked_fixture_tuple_list):
        serv = self._create_cmd_service(self.service_class, is_save_state=True)
        _, fixtures = self.run_function_with_mocks(
            serv.run,
            mocked_fixture_tuple_list
        )
        for item in self.response[self.service_name]:
            self.assertTrue(item['keypair']['name'],
                            serv.data[self.service_name])
        for fixture in fixtures:
            fixture.mock.assert_called_once()

    def test_delete_fail(self):
        delete_mock = [(self.get_method, self.response, 200),
                       (self.validate_response, 'validate', None),
                       (self.delete_method, 'error', None),
                       (self.log_method, 'exception', None)]
        self._test_delete(delete_mock, fail=True)

    def test_delete_pass(self):
        delete_mock = [(self.get_method, self.response, 200),
                       (self.validate_response, 'validate', None),
                       (self.delete_method, None, 204),
                       (self.log_method, 'exception', None)]
        self._test_delete(delete_mock)

    def test_dry_run(self):
        dry_mock = [(self.get_method, self.response, 200),
                    (self.validate_response, 'validate', None),
                    (self.delete_method, "delete", None)]
        self._test_dry_run_true(dry_mock)

    def test_save_state(self):
        self._test_saved_state_true([
            (self.get_method, self.response, 200),
            (self.validate_response, 'validate', None)
        ])

    def test_prefix_opt_precedence(self):
        delete_mock = [(self.filter_saved_state, [], None),
                       (self.filter_resource_list, [], None),
                       (self.filter_prefix, [], None),
                       (self.get_method, self.response, 200),
                       (self.validate_response, 'validate', None),
                       (self.delete_method, 'error', None),
                       (self.log_method, 'exception', None)]
        self._test_prefix_opt_precedence(delete_mock)

    def test_resource_list_opt_precedence(self):
        delete_mock = [(self.filter_saved_state, [], None),
                       (self.filter_resource_list, [], None),
                       (self.filter_prefix, [], None),
                       (self.get_method, self.response, 200),
                       (self.validate_response, 'validate', None),
                       (self.delete_method, 'error', None),
                       (self.log_method, 'exception', None)]
        self._test_resource_list_opt_precedence(delete_mock)


class TestVolumeService(BaseCmdServiceTests):

    service_class = 'VolumeService'
    service_name = 'volumes'
    response = {
        "volumes": [
            {
                "id": "efa54464-8fab-47cd-a05a-be3e6b396188",
                "links": [
                    {
                        "href": "http://127.0.0.1:37097/v3/89af/volumes/efa54",
                        "rel": "self"
                    },
                    {
                        "href": "http://127.0.0.1:37097/89af/volumes/efa54464",
                        "rel": "bookmark"
                    }
                ],
                "name": "volume-name"
            },
            {
                "id": "aa77asdf-1234",
                "name": "saved-volume",
                "links": [],
            }
        ]
    }

    def test_delete_fail(self):
        delete_mock = [(self.get_method, self.response, 200),
                       (self.delete_method, 'error', None),
                       (self.log_method, 'exception', None)]
        self._test_delete(delete_mock, fail=True)

    def test_delete_pass(self):
        delete_mock = [(self.get_method, self.response, 200),
                       (self.delete_method, None, 202),
                       (self.log_method, 'exception', None)]
        self._test_delete(delete_mock)

    def test_dry_run(self):
        dry_mock = [(self.get_method, self.response, 200),
                    (self.delete_method, "delete", None)]
        self._test_dry_run_true(dry_mock)

    def test_save_state(self):
        self._test_saved_state_true([(self.get_method, self.response, 200)])

    def test_prefix_opt_precedence(self):
        delete_mock = [(self.filter_saved_state, [], None),
                       (self.filter_resource_list, [], None),
                       (self.filter_prefix, [], None),
                       (self.get_method, self.response, 200),
                       (self.delete_method, 'error', None),
                       (self.log_method, 'exception', None)]
        self._test_prefix_opt_precedence(delete_mock)

    def test_resource_list_opt_precedence(self):
        delete_mock = [(self.filter_saved_state, [], None),
                       (self.filter_resource_list, [], None),
                       (self.filter_prefix, [], None),
                       (self.get_method, self.response, 200),
                       (self.delete_method, 'error', None),
                       (self.log_method, 'exception', None)]
        self._test_resource_list_opt_precedence(delete_mock)


class TestVolumeQuotaService(BaseCmdServiceTests):

    service_class = 'VolumeQuotaService'
    service_name = 'volume_quota_service'
    response = {
        "quota_set": {
            "groups":
                {"reserved": 0, "limit": 10, "in_use": 0},
            "per_volume_gigabytes":
                {"reserved": 0, "limit": -1, "in_use": 0},
            "volumes":
                {"reserved": 0, "limit": 10, "in_use": 0},
            "gigabytes":
                {"reserved": 0, "limit": 1000, "in_use": 0},
            "backup_gigabytes":
                {"reserved": 0, "limit": 1000, "in_use": 0},
            "snapshots":
                {"reserved": 0, "limit": 10, "in_use": 0},
            "volumes_iscsi":
                {"reserved": 0, "limit": -1, "in_use": 0},
            "snapshots_iscsi":
                {"reserved": 0, "limit": -1, "in_use": 0},
            "backups":
                {"reserved": 0, "limit": 10, "in_use": 0},
            "gigabytes_iscsi":
                {"reserved": 0, "limit": -1, "in_use": 0},
            "id": "b8e3ece07bb049138d224436756e3b57"
                }
        }

    def test_delete_fail(self):
        delete_mock = [(self.delete_method, 'error', None),
                       (self.log_method, 'exception', None)]
        self._test_delete(delete_mock, fail=True)

    def test_delete_pass(self):
        delete_mock = [(self.delete_method, None, 200),
                       (self.log_method, 'exception', None)]
        self._test_delete(delete_mock)

    def test_dry_run(self):
        dry_mock = [(self.get_method, self.response, 200),
                    (self.delete_method, "delete", None)]
        self._test_dry_run_true(dry_mock)


class TestNovaQuotaService(BaseCmdServiceTests):

    service_class = 'NovaQuotaService'
    service_name = 'nova_quota_service'
    response = {
        "limits": {
            "rate": [],
            "absolute": {
                "maxServerMeta": 128,
                "maxPersonality": 5,
                "totalServerGroupsUsed": 0,
                "maxImageMeta": 128,
                "maxPersonalitySize": 10240,
                "maxTotalKeypairs": 100,
                "maxSecurityGroupRules": 20,
                "maxServerGroups": 10,
                "totalCoresUsed": 0,
                "totalRAMUsed": 0,
                "totalInstancesUsed": 0,
                "maxSecurityGroups": 10,
                "totalFloatingIpsUsed": 0,
                "maxTotalCores": 20,
                "maxServerGroupMembers": 10,
                "maxTotalFloatingIps": 10,
                "totalSecurityGroupsUsed": 0,
                "maxTotalInstances": 10,
                "maxTotalRAMSize": 51200
                }
            }
        }

    def test_delete_fail(self):
        delete_mock = [(self.delete_method, 'error', None),
                       (self.log_method, 'exception', None)]
        self._test_delete(delete_mock, fail=True)

    def test_delete_pass(self):
        delete_mock = [(self.delete_method, None, 202),
                       (self.log_method, 'exception', None)]
        self._test_delete(delete_mock)

    def test_dry_run(self):
        dry_mock = [(self.get_method, self.response, 200),
                    (self.delete_method, "delete", None)]
        self._test_dry_run_true(dry_mock)


class TestNetworkQuotaService(BaseCmdServiceTests):

    service_class = 'NetworkQuotaService'
    service_name = 'network_quota_service'
    response = {
        "quotas": [{
            "subnet": 110,
            "network": 100,
            "floatingip": 50,
            "tenant_id": "81e8490db559474dacb2212fca9cca2d",
            "subnetpool": -1,
            "security_group_rule": 100,
            "trunk": -1,
            "security_group": 10,
            "router": 10,
            "rbac_policy": 10, "project_id":
            "81e8490db559474dacb2212fca9cca2d", "port": 500
            }]
    }

    def test_delete_fail(self):
        delete_mock = [(self.delete_method, 'error', None),
                       (self.log_method, 'exception', None)]
        self._test_delete(delete_mock, fail=True)

    def test_delete_pass(self):
        delete_mock = [(self.delete_method, None, 204),
                       (self.log_method, 'exception', None)]
        self._test_delete(delete_mock)

    def test_dry_run(self):
        dry_mock = [(self.get_method, self.response, 200),
                    (self.delete_method, "delete", None)]
        self._test_dry_run_true(dry_mock)


# Begin network service classes
class TestNetworkService(BaseCmdServiceTests):

    service_class = 'NetworkService'
    service_name = 'networks'
    response = {
        "networks": [
            {
                "admin_state_up": True,
                "availability_zone_hints": [],
                "availability_zones": [
                    "nova"
                ],
                "created_at": "2016-03-08T20:19:41",
                "dns_domain": "my-domain.org.",
                "id": "d32019d3-bc6e-4319-9c1d-6722fc136a22",
                "l2_adjacency": False,
                "mtu": 1500,
                "name": "net1",
                "port_security_enabled": True,
                "project_id": "4fd44f30292945e481c7b8a0c8908869",
                "qos_policy_id": "6a8454ade84346f59e8d40665f878b2e",
                "revision_number": 1,
                "router:external": False,
                "shared": False,
                "status": "ACTIVE",
                "subnets": [
                    "54d6f61d-db07-451c-9ab3-b9609b6b6f0b"
                ],
                "tenant_id": "4fd44f30292945e481c7b8a0c8908869",
                "updated_at": "2016-03-08T20:19:41",
                "vlan_transparent": True,
                "description": "",
                "is_default": False
            },
            {
                "id": "6722fc13-4319",
                "name": "saved-network"
            }
        ]
    }

    def test_delete_fail(self):
        delete_mock = [(self.get_method, self.response, 200),
                       (self.delete_method, 'error', None),
                       (self.log_method, 'exception', None)]
        self._test_delete(delete_mock, fail=True)

    def test_delete_pass(self):
        delete_mock = [(self.get_method, self.response, 200),
                       (self.delete_method, None, 204),
                       (self.log_method, 'exception', None)]
        self._test_delete(delete_mock)

    def test_dry_run(self):
        dry_mock = [(self.get_method, self.response, 200),
                    (self.delete_method, "delete", None)]
        self._test_dry_run_true(dry_mock)

    def test_save_state(self):
        self._test_saved_state_true([(self.get_method, self.response, 200)])

    def test_preserve_list(self):
        self.response['networks'].append(
            {
                "admin_state_up": True,
                "availability_zone_hints": [],
                "availability_zones": [
                    "nova"
                ],
                "created_at": "2017-03-08T20:19:41",
                "dns_domain": "my-domain.org.",
                "id": cleanup_service.CONF_PUB_NETWORK,
                "name": "net2",
                "port_security_enabled": True,
                "project_id": "4fd44f30292945e481c7b8a0c8908869",
                "qos_policy_id": "6a8454ade84346f59e8d40665f878b2e",
                "revision_number": 1,
                "status": "ACTIVE",
                "subnets": [
                    "54d6f61d-db07-451c-9ab3-b9609b6b6f0b"
                ],
                "tenant_id": "4fd44f30292945e481c7b8a0c8908869",
                "updated_at": "2018-03-08T20:19:41",
                "vlan_transparent": True,
                "is_default": False
            })
        self._test_is_preserve_true([(self.get_method, self.response, 200)])

    def test_prefix_opt_precedence(self):
        delete_mock = [(self.filter_saved_state, [], None),
                       (self.filter_resource_list, [], None),
                       (self.filter_prefix, [], None),
                       (self.get_method, self.response, 200),
                       (self.delete_method, 'error', None),
                       (self.log_method, 'exception', None)]
        self._test_prefix_opt_precedence(delete_mock)

    def test_resource_list_opt_precedence(self):
        delete_mock = [(self.filter_saved_state, [], None),
                       (self.filter_resource_list, [], None),
                       (self.filter_prefix, [], None),
                       (self.get_method, self.response, 200),
                       (self.delete_method, 'error', None),
                       (self.log_method, 'exception', None)]
        self._test_resource_list_opt_precedence(delete_mock)


class TestNetworkFloatingIpService(BaseCmdServiceTests):

    service_class = 'NetworkFloatingIpService'
    service_name = 'floatingips'
    response = {
        "floatingips": [
            {
                "router_id": "d23abc8d-2991-4a55-ba98-2aaea84cc72f",
                "description": "for test",
                "dns_domain": "my-domain.org.",
                "dns_name": "myfip",
                "created_at": "2016-12-21T10:55:50Z",
                "updated_at": "2016-12-21T10:55:53Z",
                "revision_number": 1,
                "project_id": "4969c491a3c74ee4af974e6d800c62de",
                "tenant_id": "4969c491a3c74ee4af974e6d800c62de",
                "floating_network_id": "376da547-b977-4cfe-9cba-275c80debf57",
                "fixed_ip_address": "10.0.0.3",
                "floating_ip_address": "172.24.4.228",
                "port_id": "ce705c24-c1ef-408a-bda3-7bbd946164ab",
                "id": "2f245a7b-796b-4f26-9cf9-9e82d248fda7",
                "status": "ACTIVE",
                "port_details": {
                    "status": "ACTIVE",
                    "name": "",
                    "admin_state_up": True,
                    "network_id": "02dd8479-ef26-4398-a102-d19d0a7b3a1f",
                    "device_owner": "compute:nova",
                    "mac_address": "fa:16:3e:b1:3b:30",
                    "device_id": "8e3941b4-a6e9-499f-a1ac-2a4662025cba"
                },
                "tags": ["tag1,tag2"],
                "port_forwardings": []
            },
            {
                "id": "9e82d248-408a",
                "status": "ACTIVE"
            }
        ]
    }

    def test_delete_fail(self):
        delete_mock = [(self.get_method, self.response, 200),
                       (self.delete_method, 'error', None),
                       (self.log_method, 'exception', None)]
        self._test_delete(delete_mock, fail=True)

    def test_delete_pass(self):
        delete_mock = [(self.get_method, self.response, 200),
                       (self.delete_method, None, 204),
                       (self.log_method, 'exception', None)]
        self._test_delete(delete_mock)

    def test_dry_run(self):
        dry_mock = [(self.get_method, self.response, 200),
                    (self.delete_method, "delete", None)]
        self._test_dry_run_true(dry_mock)

    def test_save_state(self):
        self._test_saved_state_true([(self.get_method, self.response, 200)])

    def test_prefix_opt_precedence(self):
        delete_mock = [(self.filter_saved_state, [], None),
                       (self.filter_resource_list, [], None),
                       (self.filter_prefix, [], None),
                       (self.get_method, self.response, 200),
                       (self.delete_method, 'error', None),
                       (self.log_method, 'exception', None)]
        serv = self._create_cmd_service(
            self.service_class, is_resource_list=True, prefix='tempest')
        _, fixtures = self.run_function_with_mocks(
            serv.run,
            delete_mock
        )

        # cleanup returns []
        fixtures[0].mock.assert_not_called()
        fixtures[1].mock.assert_not_called()
        fixtures[2].mock.assert_not_called()

    def test_resource_list_opt_precedence(self):
        delete_mock = [(self.filter_saved_state, [], None),
                       (self.filter_resource_list, [], None),
                       (self.filter_prefix, [], None),
                       (self.get_method, self.response, 200),
                       (self.delete_method, 'error', None),
                       (self.log_method, 'exception', None)]
        self._test_resource_list_opt_precedence(delete_mock)


class TestNetworkRouterService(BaseCmdServiceTests):

    service_class = 'NetworkRouterService'
    service_name = 'routers'
    validate_response = ('tempest.lib.services.network.routers_client'
                         '.RoutersClient.validate_response')
    response = {
        "routers": [
            {
                "admin_state_up": True,
                "availability_zone_hints": [],
                "availability_zones": [
                    "nova"
                ],
                "created_at": "2018-03-19T19:17:04Z",
                "description": "",
                "distributed": False,
                "external_gateway_info": {
                    "enable_snat": True,
                    "external_fixed_ips": [
                        {
                            "ip_address": "172.24.4.3",
                            "subnet_id": "b930d7f6-ceb7-40a0-8b81-a425dd994ccf"
                        },
                        {
                            "ip_address": "2001:db8::c",
                            "subnet_id": "0c56df5d-ace5-46c8-8f4c-45fa4e334d18"
                        }
                    ],
                    "network_id": "ae34051f-aa6c-4c75-abf5-50dc9ac99ef3"
                },
                "flavor_id": "f7b14d9a-b0dc-4fbe-bb14-a0f4970a69e0",
                "ha": False,
                "id": "915a14a6-867b-4af7-83d1-70efceb146f9",
                "name": "router2",
                "revision_number": 1,
                "routes": [
                    {
                        "destination": "179.24.1.0/24",
                        "nexthop": "172.24.3.99"
                    }
                ],
                "status": "ACTIVE",
                "updated_at": "2018-03-19T19:17:22Z",
                "project_id": "0bd18306d801447bb457a46252d82d13",
                "tenant_id": "0bd18306d801447bb457a46252d82d13",
                "tags": ["tag1,tag2"]
            },
            {
                "id": "4s5w34hj-id44",
                "name": "saved-router"
            }
        ],
        # "ports" key is added to the response in order to simplify unit
        # testing - it's because NetworkRouterService's delete method lists
        # ports before deleting any router
        "ports": []
    }

    def _test_delete(self, mocked_fixture_tuple_list, fail=False):
        serv = self._create_cmd_service(self.service_class)
        resp, fixtures = self.run_function_with_mocks(
            serv.run,
            mocked_fixture_tuple_list,
        )
        for fixture in fixtures:
            if fail is False and fixture.mock.return_value == 'exception':
                fixture.mock.assert_not_called()
            elif self.service_name in self.saved_state.keys():
                fixture.mock.assert_called()
                for key in self.saved_state[self.service_name].keys():
                    self.assertNotIn(key, fixture.mock.call_args[0][0])
        self.assertFalse(serv.data)

    def test_delete_fail(self):
        delete_mock = [(self.get_method, self.response, 200),
                       (self.delete_method, 'error', None),
                       (self.log_method, 'exception', None)]
        self._test_delete(delete_mock, fail=True)

    def test_delete_pass(self):
        delete_mock = [(self.get_method, self.response, 200),
                       (self.delete_method, None, 204),
                       (self.log_method, 'exception', None)]
        self._test_delete(delete_mock)

    def test_dry_run(self):
        dry_mock = [(self.get_method, self.response, 200),
                    (self.delete_method, "delete", None)]
        self._test_dry_run_true(dry_mock)

    def test_save_state(self):
        self._test_saved_state_true([(self.get_method, self.response, 200)])

    def test_preserve_list(self):
        self.response['routers'].append(
            {
                "admin_state_up": True,
                "availability_zone_hints": [],
                "availability_zones": [
                    "nova"
                ],
                "created_at": "2018-03-19T19:17:04Z",
                "id": cleanup_service.CONF_PUB_ROUTER,
                "name": "router-preserve",
                "status": "ACTIVE",
                "updated_at": "2018-03-19T19:17:22Z",
                "project_id": "0bd18306d801447bb457a46252d82d13",
                "tenant_id": "0bd18306d801447bb457a46252d82d13",
                "tags": ["tag1,tag2"]
            })
        self._test_is_preserve_true([(self.get_method, self.response, 200)])

    def test_prefix_opt_precedence(self):
        delete_mock = [(self.filter_saved_state, [], None),
                       (self.filter_resource_list, [], None),
                       (self.filter_prefix, [], None),
                       (self.get_method, self.response, 200),
                       (self.delete_method, 'error', None),
                       (self.log_method, 'exception', None)]
        self._test_prefix_opt_precedence(delete_mock)

    def test_resource_list_opt_precedence(self):
        delete_mock = [(self.filter_saved_state, [], None),
                       (self.filter_resource_list, [], None),
                       (self.filter_prefix, [], None),
                       (self.get_method, self.response, 200),
                       (self.delete_method, 'error', None),
                       (self.log_method, 'exception', None)]
        self._test_resource_list_opt_precedence(delete_mock)


class TestNetworkMeteringLabelRuleService(BaseCmdServiceTests):

    service_class = 'NetworkMeteringLabelRuleService'
    service_name = 'metering_label_rules'
    response = {
        "metering_label_rules": [
            {
                "remote_ip_prefix": "20.0.0.0/24",
                "direction": "ingress",
                "metering_label_id": "e131d186-b02d-4c0b-83d5-0c0725c4f812",
                "id": "9536641a-7d14-4dc5-afaf-93a973ce0eb8",
                "excluded": False
            },
            {
                "direction": "ingress",
                "id": "93a973ce-4dc5"
            }
        ]
    }

    def test_delete_fail(self):
        delete_mock = [(self.get_method, self.response, 200),
                       (self.delete_method, 'error', None),
                       (self.log_method, 'exception', None)]
        self._test_delete(delete_mock, fail=True)

    def test_delete_pass(self):
        delete_mock = [(self.get_method, self.response, 200),
                       (self.delete_method, None, 204),
                       (self.log_method, 'exception', None)]
        self._test_delete(delete_mock)

    def test_dry_run(self):
        dry_mock = [(self.get_method, self.response, 200),
                    (self.delete_method, "delete", None)]
        self._test_dry_run_true(dry_mock)

    def test_save_state(self):
        self._test_saved_state_true([(self.get_method, self.response, 200)])

    def test_prefix_opt_precedence(self):
        delete_mock = [(self.filter_saved_state, [], None),
                       (self.filter_resource_list, [], None),
                       (self.filter_prefix, [], None),
                       (self.get_method, self.response, 200),
                       (self.delete_method, 'error', None),
                       (self.log_method, 'exception', None)]
        serv = self._create_cmd_service(
            self.service_class, is_resource_list=True, prefix='tempest')
        _, fixtures = self.run_function_with_mocks(
            serv.run,
            delete_mock
        )

        # cleanup returns []
        fixtures[0].mock.assert_not_called()
        fixtures[1].mock.assert_not_called()
        fixtures[2].mock.assert_not_called()

    def test_resource_list_opt_precedence(self):
        delete_mock = [(self.filter_saved_state, [], None),
                       (self.filter_resource_list, [], None),
                       (self.filter_prefix, [], None),
                       (self.get_method, self.response, 200),
                       (self.delete_method, 'error', None),
                       (self.log_method, 'exception', None)]
        self._test_resource_list_opt_precedence(delete_mock)


class TestNetworkMeteringLabelService(BaseCmdServiceTests):

    service_class = 'NetworkMeteringLabelService'
    service_name = 'metering_labels'
    response = {
        "metering_labels": [
            {
                "project_id": "45345b0ee1ea477fac0f541b2cb79cd4",
                "tenant_id": "45345b0ee1ea477fac0f541b2cb79cd4",
                "description": "label1 description",
                "name": "label1",
                "id": "a6700594-5b7a-4105-8bfe-723b346ce866",
                "shared": False
            },
            {
                "name": "saved-label",
                "id": "723b346ce866-4c7q",
            }
        ]
    }

    def test_delete_fail(self):
        delete_mock = [(self.get_method, self.response, 200),
                       (self.delete_method, 'error', None),
                       (self.log_method, 'exception', None)]
        self._test_delete(delete_mock, fail=True)

    def test_delete_pass(self):
        delete_mock = [(self.get_method, self.response, 200),
                       (self.delete_method, None, 204),
                       (self.log_method, 'exception', None)]
        self._test_delete(delete_mock)

    def test_dry_run(self):
        dry_mock = [(self.get_method, self.response, 200),
                    (self.delete_method, "delete", None)]
        self._test_dry_run_true(dry_mock)

    def test_save_state(self):
        self._test_saved_state_true([(self.get_method, self.response, 200)])

    def test_prefix_opt_precedence(self):
        delete_mock = [(self.filter_saved_state, [], None),
                       (self.filter_resource_list, [], None),
                       (self.filter_prefix, [], None),
                       (self.get_method, self.response, 200),
                       (self.delete_method, 'error', None),
                       (self.log_method, 'exception', None)]
        self._test_prefix_opt_precedence(delete_mock)

    def test_resource_list_opt_precedence(self):
        delete_mock = [(self.filter_saved_state, [], None),
                       (self.filter_resource_list, [], None),
                       (self.filter_prefix, [], None),
                       (self.get_method, self.response, 200),
                       (self.delete_method, 'error', None),
                       (self.log_method, 'exception', None)]
        self._test_resource_list_opt_precedence(delete_mock)


class TestNetworkPortService(BaseCmdServiceTests):

    service_class = 'NetworkPortService'
    service_name = 'ports'
    response = {
        "ports": [
            {
                "admin_state_up": True,
                "allowed_address_pairs": [],
                "created_at": "2016-03-08T20:19:41",
                "description": "",
                "device_id": "9ae135f4-b6e0-4dad-9e91-3c223e385824",
                "device_owner": "",
                "dns_assignment": {
                    "hostname": "myport",
                    "ip_address": "172.24.4.2",
                    "fqdn": "myport.my-domain.org"
                },
                "dns_domain": "my-domain.org.",
                "dns_name": "myport",
                "extra_dhcp_opts": [
                    {
                        "opt_value": "pxelinux.0",
                        "ip_version": 4,
                        "opt_name": "bootfile-name"
                    }
                ],
                "fixed_ips": [
                    {
                        "ip_address": "172.24.4.2",
                        "subnet_id": "008ba151-0b8c-4a67-98b5-0d2b87666062"
                    }
                ],
                "id": "d80b1a3b-4fc1-49f3-952e-1e2ab7081d8b",
                "ip_allocation": "immediate",
                "mac_address": "fa:16:3e:58:42:ed",
                "name": "test_port",
                "network_id": "70c1db1f-b701-45bd-96e0-a313ee3430b3",
                "project_id": "",
                "revision_number": 1,
                "security_groups": [],
                "status": "ACTIVE",
                "tags": ["tag1,tag2"],
                "tenant_id": "",
                "updated_at": "2016-03-08T20:19:41",
                "qos_policy_id": "29d5e02e-d5ab-4929-bee4-4a9fc12e22ae",
                "port_security_enabled": False
            },
            {
                "id": "aa74aa4v-741a",
                "name": "saved-port",
                "device_owner": ""
            }
        ]
    }

    def test_delete_fail(self):
        delete_mock = [(self.get_method, self.response, 200),
                       (self.delete_method, 'error', None),
                       (self.log_method, 'exception', None)]
        self._test_delete(delete_mock, fail=True)

    def test_delete_pass(self):
        delete_mock = [(self.get_method, self.response, 200),
                       (self.delete_method, None, 204),
                       (self.log_method, 'exception', None)]
        self._test_delete(delete_mock)

    def test_dry_run(self):
        dry_mock = [(self.get_method, self.response, 200),
                    (self.delete_method, "delete", None)]
        self._test_dry_run_true(dry_mock)

    def test_save_state(self):
        self._test_saved_state_true([(self.get_method, self.response, 200)])

    def test_preserve_list(self):
        self.response['ports'].append(
            {
                "created_at": "2018-03-08T20:19:41",
                "description": "",
                "device_id": "9ae135f4-b6e0-4dad-9e91-3c223e385824",
                "device_owner": "compute:router_gateway",
                "id": "d80b1a3b-4fc1-49f3-952e-1fdy1ws542",
                "ip_allocation": "immediate",
                "mac_address": "fa:16:3e:58:42:ed",
                "name": "preserve_port",
                "network_id": cleanup_service.CONF_PUB_NETWORK,
                "project_id": "",
                "security_groups": [],
                "status": "ACTIVE",
                "tags": ["tag1,tag2"],
                "tenant_id": "",
                "updated_at": "2018-03-08T20:19:41",
            })
        self._test_is_preserve_true([(self.get_method, self.response, 200)])

    def test_prefix_opt_precedence(self):
        delete_mock = [(self.filter_saved_state, [], None),
                       (self.filter_resource_list, [], None),
                       (self.filter_prefix, [], None),
                       (self.get_method, self.response, 200),
                       (self.delete_method, 'error', None),
                       (self.log_method, 'exception', None)]
        self._test_prefix_opt_precedence(delete_mock)

    def test_resource_list_opt_precedence(self):
        delete_mock = [(self.filter_saved_state, [], None),
                       (self.filter_resource_list, [], None),
                       (self.filter_prefix, [], None),
                       (self.get_method, self.response, 200),
                       (self.delete_method, 'error', None),
                       (self.log_method, 'exception', None)]
        self._test_resource_list_opt_precedence(delete_mock)


class TestNetworkSecGroupService(BaseCmdServiceTests):

    service_class = 'NetworkSecGroupService'
    service_name = 'security_groups'
    response = {
        "security_groups": [
            {
                "description": "default",
                "id": "85cc3048-abc3-43cc-89b3-377341426ac5",
                "name": "test",
                "security_group_rules": [
                    {
                        "direction": "egress",
                        "ethertype": "IPv6",
                        "id": "3c0e45ff-adaf-4124-b083-bf390e5482ff",
                        "security_group_id": "85cc3048-abc3-43cc-89b3-3773414",
                        "project_id": "e4f50856753b4dc6afee5fa6b9b6c550",
                        "revision_number": 1,
                        "tags": ["tag1,tag2"],
                        "tenant_id": "e4f50856753b4dc6afee5fa6b9b6c550",
                        "created_at": "2018-03-19T19:16:56Z",
                        "updated_at": "2018-03-19T19:16:56Z",
                        "description": ""
                    }
                ]
            },
            {
                "id": "7q844add-3697",
                "name": "saved-sec-group"
            }
        ]
    }

    def test_delete_fail(self):
        delete_mock = [(self.get_method, self.response, 200),
                       (self.delete_method, 'error', None),
                       (self.log_method, 'exception', None)]
        self._test_delete(delete_mock, fail=True)

    def test_delete_pass(self):
        delete_mock = [(self.get_method, self.response, 200),
                       (self.delete_method, None, 204),
                       (self.log_method, 'exception', None)]
        self._test_delete(delete_mock)

    def test_dry_run(self):
        dry_mock = [(self.get_method, self.response, 200),
                    (self.delete_method, "delete", None)]
        self._test_dry_run_true(dry_mock)

    def test_save_state(self):
        self._test_saved_state_true([(self.get_method, self.response, 200)])

    def test_preserve_list(self):
        self.response['security_groups'].append(
            {
                "description": "default",
                "id": "85cc3048-abc3-43cc-89b3-377341426ac5",
                "name": "test",
                "security_group_rules": [
                    {
                        "direction": "egress",
                        "ethertype": "IPv6",
                        "id": "3c0e45ff-adaf-4124-b083-bf390e5482ff",
                        "security_group_id": "85cc3048-abc3-43cc-89b3-3773414",
                        "project_id": cleanup_service.CONF_PROJECTS[0],
                        "revision_number": 1,
                        "tags": ["tag1,tag2"],
                        "tenant_id": "e4f50856753b4dc6afee5fa6b9b6c550",
                        "created_at": "2018-03-19T19:16:56Z",
                        "updated_at": "2018-03-19T19:16:56Z",
                        "description": ""
                    }
                ]
            })
        self._test_is_preserve_true([(self.get_method, self.response, 200)])

    def test_prefix_opt_precedence(self):
        delete_mock = [(self.filter_saved_state, [], None),
                       (self.filter_resource_list, [], None),
                       (self.filter_prefix, [], None),
                       (self.get_method, self.response, 200),
                       (self.delete_method, 'error', None),
                       (self.log_method, 'exception', None)]
        self._test_prefix_opt_precedence(delete_mock)

    def test_resource_list_opt_precedence(self):
        delete_mock = [(self.filter_saved_state, [], None),
                       (self.filter_resource_list, [], None),
                       (self.filter_prefix, [], None),
                       (self.get_method, self.response, 200),
                       (self.delete_method, 'error', None),
                       (self.log_method, 'exception', None)]
        self._test_resource_list_opt_precedence(delete_mock)


class TestNetworkSubnetService(BaseCmdServiceTests):

    service_class = 'NetworkSubnetService'
    service_name = 'subnets'
    response = {
        "subnets": [
            {
                "name": "private-subnet",
                "enable_dhcp": True,
                "network_id": "db193ab3-96e3-4cb3-8fc5-05f4296d0324",
                "project_id": "26a7980765d0414dbc1fc1f88cdb7e6e",
                "tenant_id": "26a7980765d0414dbc1fc1f88cdb7e6e",
                "dns_nameservers": [],
                "allocation_pools": [
                    {
                        "start": "10.0.0.2",
                        "end": "10.0.0.254"
                    }
                ],
                "host_routes": [],
                "ip_version": 4,
                "gateway_ip": "10.0.0.1",
                "cidr": "10.0.0.0/24",
                "id": "08eae331-0402-425a-923c-34f7cfe39c1b",
                "created_at": "2016-10-10T14:35:34Z",
                "revision_number": 2,
                "service_types": [],
                "tags": ["tag1,tag2"],
                "updated_at": "2016-10-10T14:35:34Z"
            },
            {
                "id": "55ttda4a-2584",
                "name": "saved-subnet"
            }
        ]
    }

    def test_delete_fail(self):
        delete_mock = [(self.get_method, self.response, 200),
                       (self.delete_method, 'error', None),
                       (self.log_method, 'exception', None)]
        self._test_delete(delete_mock, fail=True)

    def test_delete_pass(self):
        delete_mock = [(self.get_method, self.response, 200),
                       (self.delete_method, None, 204),
                       (self.log_method, 'exception', None)]
        self._test_delete(delete_mock)

    def test_dry_run(self):
        dry_mock = [(self.get_method, self.response, 200),
                    (self.delete_method, "delete", None)]
        self._test_dry_run_true(dry_mock)

    def test_save_state(self):
        self._test_saved_state_true([(self.get_method, self.response, 200)])

    def test_preserve_list(self):
        self.response['subnets'].append(
            {
                "name": "public-subnet",
                "network_id": cleanup_service.CONF_PUB_NETWORK,
                "project_id": "26a7980765d0414dbc1fc1f88cdb7e6e",
                "tenant_id": "26a7980765d0414dbc1fc1f88cdb7e6e",
                "ip_version": 4,
                "gateway_ip": "10.0.0.1",
                "cidr": "10.0.0.0/24",
                "id": "08eae331-0402-425a-923c-34f7cfe39c1b",
                "created_at": "2018-10-10T14:35:34Z",
                "service_types": [],
                "tags": ["tag1,tag2"],
                "updated_at": "2018-10-10T14:35:34Z"
            })
        self._test_is_preserve_true([(self.get_method, self.response, 200)])

    def test_prefix_opt_precedence(self):
        delete_mock = [(self.filter_saved_state, [], None),
                       (self.filter_resource_list, [], None),
                       (self.filter_prefix, [], None),
                       (self.get_method, self.response, 200),
                       (self.delete_method, 'error', None),
                       (self.log_method, 'exception', None)]
        self._test_prefix_opt_precedence(delete_mock)

    def test_resource_list_opt_precedence(self):
        delete_mock = [(self.filter_saved_state, [], None),
                       (self.filter_resource_list, [], None),
                       (self.filter_prefix, [], None),
                       (self.get_method, self.response, 200),
                       (self.delete_method, 'error', None),
                       (self.log_method, 'exception', None)]
        self._test_resource_list_opt_precedence(delete_mock)


class TestNetworkSubnetPoolsService(BaseCmdServiceTests):

    service_class = 'NetworkSubnetPoolsService'
    service_name = 'subnetpools'
    response = {
        "subnetpools": [
            {
                "min_prefixlen": "64",
                "default_prefixlen": "64",
                "id": "03f761e6-eee0-43fc-a921-8acf64c14988",
                "max_prefixlen": "64",
                "name": "my-subnet-pool-ipv6",
                "is_default": False,
                "project_id": "9fadcee8aa7c40cdb2114fff7d569c08",
                "tenant_id": "9fadcee8aa7c40cdb2114fff7d569c08",
                "prefixes": [
                    "2001:db8:0:2::/64",
                    "2001:db8::/63"
                ],
                "ip_version": 6,
                "shared": False,
                "description": "",
                "created_at": "2016-03-08T20:19:41",
                "updated_at": "2016-03-08T20:19:41",
                "revision_number": 2,
                "tags": ["tag1,tag2"]
            },
            {
                "id": "8acf64c1-43fc",
                "name": "saved-subnet-pool"
            }
        ]
    }

    def test_delete_fail(self):
        delete_mock = [(self.get_method, self.response, 200),
                       (self.delete_method, 'error', None),
                       (self.log_method, 'exception', None)]
        self._test_delete(delete_mock, fail=True)

    def test_delete_pass(self):
        delete_mock = [(self.get_method, self.response, 200),
                       (self.delete_method, None, 204),
                       (self.log_method, 'exception', None)]
        self._test_delete(delete_mock)

    def test_dry_run(self):
        dry_mock = [(self.get_method, self.response, 200),
                    (self.delete_method, "delete", None)]
        self._test_dry_run_true(dry_mock)

    def test_save_state(self):
        self._test_saved_state_true([(self.get_method, self.response, 200)])

    def test_preserve_list(self):
        self.response['subnetpools'].append(
            {
                "min_prefixlen": "64",
                "default_prefixlen": "64",
                "id": "9acf64c1-43fc",
                "name": "preserve-pool",
                "project_id": cleanup_service.CONF_PROJECTS[0],
                "created_at": "2016-03-08T20:19:41",
                "updated_at": "2016-03-08T20:19:41"
            })
        self._test_is_preserve_true([(self.get_method, self.response, 200)])

    def test_prefix_opt_precedence(self):
        delete_mock = [(self.filter_saved_state, [], None),
                       (self.filter_resource_list, [], None),
                       (self.filter_prefix, [], None),
                       (self.get_method, self.response, 200),
                       (self.delete_method, 'error', None),
                       (self.log_method, 'exception', None)]
        self._test_prefix_opt_precedence(delete_mock)

    def test_resource_list_opt_precedence(self):
        delete_mock = [(self.filter_saved_state, [], None),
                       (self.filter_resource_list, [], None),
                       (self.filter_prefix, [], None),
                       (self.get_method, self.response, 200),
                       (self.delete_method, 'error', None),
                       (self.log_method, 'exception', None)]
        self._test_resource_list_opt_precedence(delete_mock)


# begin global services
class TestRegionService(BaseCmdServiceTests):
    service_class = 'RegionService'
    service_name = 'regions'
    response = {
        "regions": [{
            "parent_region_id": None,
            "id": "RegionOne",
            "links": {
                    "self":
                    "http://10.0.145.61:5000/v3/regions/RegionOne"
                    },
            "description": ""
        },
            {
            "parent_region_id": None,
            "id": "RegionTwo",
            "links": {
                "self":
                    "http://10.0.145.61:5000/v3/regions/RegionTwo"
                },
            "description": ""
            }],
        "links": {
            "self":
                "http://10.0.145.61:5000/v3/regions",
                "next": None,
                "previous": None
        }
    }

    def test_delete_pass(self):
        delete_mock = [(self.get_method, self.response, 200),
                       (self.delete_method, None, 204),
                       (self.log_method, "exception", None)]
        self._test_delete(delete_mock)

    def test_delete_fail(self):
        delete_mock = [(self.get_method, self.response, 200),
                       (self.delete_method, 'error', None),
                       (self.log_method, "exception", None)]
        self._test_delete(delete_mock, fail=True)

    def test_dry_run(self):
        dry_mock = [(self.get_method, self.response, 200),
                    (self.delete_method, "delete", None)]
        self._test_dry_run_true(dry_mock)

    def test_save_state(self):
        self._test_saved_state_true([(self.get_method, self.response, 200)])

    def test_prefix_opt_precedence(self):
        delete_mock = [(self.filter_saved_state, [], None),
                       (self.filter_resource_list, [], None),
                       (self.filter_prefix, [], None),
                       (self.get_method, self.response, 200),
                       (self.delete_method, 'error', None),
                       (self.log_method, 'exception', None)]
        serv = self._create_cmd_service(
            self.service_class, is_resource_list=True, prefix='tempest')
        _, fixtures = self.run_function_with_mocks(
            serv.run,
            delete_mock
        )

        # cleanup returns []
        fixtures[0].mock.assert_not_called()
        fixtures[1].mock.assert_not_called()
        fixtures[2].mock.assert_not_called()

    def test_resource_list_opt_precedence(self):
        delete_mock = [(self.filter_saved_state, [], None),
                       (self.filter_resource_list, [], None),
                       (self.filter_prefix, [], None),
                       (self.get_method, self.response, 200),
                       (self.delete_method, 'error', None),
                       (self.log_method, 'exception', None)]
        self._test_resource_list_opt_precedence(delete_mock)


class TestDomainService(BaseCmdServiceTests):

    service_class = 'DomainService'
    service_name = 'domains'
    response = {
        "domains": [
            {
                "description": "Destroy all humans",
                "enabled": True,
                "id": "5a75994a3",
                "links": {
                    "self": "http://example.com/identity/v3/domains/5a75994a3"
                },
                "name": "Sky_net"
            },
            {
                "description": "Owns users and tenants on Identity API",
                "enabled": False,
                "id": "default",
                "links": {
                    "self": "http://example.com/identity/v3/domains/default"
                },
                "name": "Default"
            }
        ]
    }

    mock_update = ("tempest.lib.services.identity.v3."
                   "domains_client.DomainsClient.update_domain")

    def test_delete_fail(self):
        delete_mock = [(self.get_method, self.response, 200),
                       (self.delete_method, 'error', None),
                       (self.log_method, 'exception', None),
                       (self.mock_update, 'update', None)]
        self._test_delete(delete_mock, fail=True)

    def test_delete_pass(self):
        delete_mock = [(self.get_method, self.response, 200),
                       (self.delete_method, None, 204),
                       (self.log_method, 'exception', None),
                       (self.mock_update, 'update', None)]
        self._test_delete(delete_mock)

    def test_dry_run(self):
        dry_mock = [(self.get_method, self.response, 200),
                    (self.delete_method, "delete", None)]
        self._test_dry_run_true(dry_mock)

    def test_save_state(self):
        self._test_saved_state_true([(self.get_method, self.response, 200)])

    def test_prefix_opt_precedence(self):
        delete_mock = [(self.filter_saved_state, [], None),
                       (self.filter_resource_list, [], None),
                       (self.filter_prefix, [], None),
                       (self.get_method, self.response, 200),
                       (self.delete_method, 'error', None),
                       (self.log_method, 'exception', None),
                       (self.mock_update, 'update', None)]
        self._test_prefix_opt_precedence(delete_mock)

    def test_resource_list_opt_precedence(self):
        delete_mock = [(self.filter_saved_state, [], None),
                       (self.filter_resource_list, [], None),
                       (self.filter_prefix, [], None),
                       (self.get_method, self.response, 200),
                       (self.delete_method, 'error', None),
                       (self.log_method, 'exception', None),
                       (self.mock_update, 'update', None)]
        self._test_resource_list_opt_precedence(delete_mock)


class TestProjectsService(BaseCmdServiceTests):

    service_class = 'ProjectService'
    service_name = 'projects'
    response = {
        "projects": [
            {
                "is_domain": False,
                "description": None,
                "domain_id": "default",
                "enabled": True,
                "id": "f38ohgp93jj032",
                "links": {
                    "self": "http://example.com/identity/v3/projects"
                            "/f38ohgp93jj032"
                },
                "name": "manhattan",
                "parent_id": None
            },
            {
                "is_domain": False,
                "description": None,
                "domain_id": "default",
                "enabled": True,
                "id": "098f89d3292ri4jf4",
                "links": {
                    "self": "http://example.com/identity/v3/projects"
                            "/098f89d3292ri4jf4"
                },
                "name": "Apollo",
                "parent_id": None
            }
        ]
    }

    def test_delete_fail(self):
        delete_mock = [(self.get_method, self.response, 200),
                       (self.delete_method, 'error', None),
                       (self.log_method, 'exception', None)]
        self._test_delete(delete_mock, fail=True)

    def test_delete_pass(self):
        delete_mock = [(self.get_method, self.response, 200),
                       (self.delete_method, None, 204),
                       (self.log_method, 'exception', None)]
        self._test_delete(delete_mock)

    def test_dry_run(self):
        dry_mock = [(self.get_method, self.response, 200),
                    (self.delete_method, "delete", None)]
        self._test_dry_run_true(dry_mock)

    def test_save_state(self):
        self._test_saved_state_true([(self.get_method, self.response, 200)])

    def test_preserve_list(self):
        self.response['projects'].append(
            {
                "is_domain": False,
                "description": None,
                "domain_id": "default",
                "enabled": True,
                "id": "r343q98h09f3092",
                "links": {
                    "self": "http://example.com/identity/v3/projects"
                            "/r343q98h09f3092"
                },
                "name": cleanup_service.CONF_PROJECTS[0],
                "parent_id": None
            })
        self._test_is_preserve_true([(self.get_method, self.response, 200)])

    def test_prefix_opt_precedence(self):
        delete_mock = [(self.filter_saved_state, [], None),
                       (self.filter_resource_list, [], None),
                       (self.filter_prefix, [], None),
                       (self.get_method, self.response, 200),
                       (self.delete_method, 'error', None),
                       (self.log_method, 'exception', None)]
        self._test_prefix_opt_precedence(delete_mock)

    def test_resource_list_opt_precedence(self):
        delete_mock = [(self.filter_saved_state, [], None),
                       (self.filter_resource_list, [], None),
                       (self.filter_prefix, [], None),
                       (self.get_method, self.response, 200),
                       (self.delete_method, 'error', None),
                       (self.log_method, 'exception', None)]
        self._test_resource_list_opt_precedence(delete_mock)


class TestImagesService(BaseCmdServiceTests):

    service_class = 'ImageService'
    service_name = 'images'
    response = {
        "images": [
            {
                "status": "ACTIVE",
                "name": "stratus-0.3.2-x86_64-disk",
                "id": "34yhwr-4t3q",
                "updated": "2014-11-03T16:40:10Z",
                "links": [{
                    "href": "http://openstack.ex.com/v2/openstack/images/"
                    "34yhwr-4t3q",
                    "rel": "self"}],
                "created": "2014-10-30T08:23:39Z",
                "minDisk": 0,
                "minRam": 0,
                "progress": 0,
                "metadata": {},
            },
            {
                "status": "ACTIVE",
                "name": "cirros-0.3.2-x86_64-disk",
                "id": "1bea47ed-f6a9",
                "updated": "2014-11-03T16:40:10Z",
                "links": [{
                    "href": "http://openstack.ex.com/v2/openstack/images/"
                    "1bea47ed-f6a9",
                    "rel": "self"}],
                "created": "2014-10-30T08:23:39Z",
                "minDisk": 0,
                "minRam": 0,
                "progress": 0,
                "metadata": {},
            }
        ]
    }

    def test_delete_fail(self):
        delete_mock = [(self.get_method, self.response, 200),
                       (self.delete_method, 'error', None),
                       (self.log_method, 'exception', None)]
        self._test_delete(delete_mock, fail=True)

    def test_delete_pass(self):
        delete_mock = [(self.get_method, self.response, 200),
                       (self.delete_method, None, 204),
                       (self.log_method, 'exception', None)]
        self._test_delete(delete_mock)

    def test_dry_run(self):
        dry_mock = [(self.get_method, self.response, 200),
                    (self.delete_method, "delete", None)]
        self._test_dry_run_true(dry_mock)

    def test_save_state(self):
        self._test_saved_state_true([(self.get_method, self.response, 200)])

    def test_preserve_list(self):
        self.response['images'].append(
            {
                "status": "ACTIVE",
                "name": "cirros-0.3.2-x86_64-disk",
                "id": cleanup_service.CONF_IMAGES[0],
                "updated": "2014-11-03T16:40:10Z",
                "links": [{
                    "href": "http://openstack.ex.com/v2/openstack/images/"
                    "None",
                    "rel": "self"}],
                "created": "2014-10-30T08:23:39Z",
                "minDisk": 0,
                "minRam": 0,
                "progress": 0,
                "metadata": {},
            })
        self._test_is_preserve_true([(self.get_method, self.response, 200)])

    def test_prefix_opt_precedence(self):
        delete_mock = [(self.filter_saved_state, [], None),
                       (self.filter_resource_list, [], None),
                       (self.filter_prefix, [], None),
                       (self.get_method, self.response, 200),
                       (self.delete_method, 'error', None),
                       (self.log_method, 'exception', None)]
        self._test_prefix_opt_precedence(delete_mock)

    def test_resource_list_opt_precedence(self):
        delete_mock = [(self.filter_saved_state, [], None),
                       (self.filter_resource_list, [], None),
                       (self.filter_prefix, [], None),
                       (self.get_method, self.response, 200),
                       (self.delete_method, 'error', None),
                       (self.log_method, 'exception', None)]
        self._test_resource_list_opt_precedence(delete_mock)


class TestFlavorService(BaseCmdServiceTests):

    service_class = 'FlavorService'
    service_name = 'flavors'
    response = {
        "flavors": [
            {
                "disk": 1,
                "id": "42",
                "links": [{
                    "href": "http://openstack.ex.com/v2/openstack/flavors/1",
                    "rel": "self"}, {
                    "href": "http://openstack.ex.com/openstack/flavors/1",
                    "rel": "bookmark"}],
                "name": "m1.tiny",
                "ram": 512,
                "swap": 1,
                "vcpus": 1
            },
            {
                "disk": 2,
                "id": "13",
                "links": [{
                    "href": "http://openstack.ex.com/v2/openstack/flavors/2",
                    "rel": "self"}, {
                    "href": "http://openstack.ex.com/openstack/flavors/2",
                    "rel": "bookmark"}],
                "name": "m1.tiny",
                "ram": 512,
                "swap": 1,
                "vcpus": 1
            }
        ]
    }

    def test_delete_fail(self):
        delete_mock = [(self.get_method, self.response, 200),
                       (self.delete_method, 'error', None),
                       (self.log_method, 'exception', None)]
        self._test_delete(delete_mock, fail=True)

    def test_delete_pass(self):
        delete_mock = [(self.get_method, self.response, 200),
                       (self.delete_method, None, 202),
                       (self.log_method, 'exception', None)]
        self._test_delete(delete_mock)

    def test_dry_run(self):
        dry_mock = [(self.get_method, self.response, 200),
                    (self.delete_method, "delete", None)]
        self._test_dry_run_true(dry_mock)

    def test_save_state(self):
        self._test_saved_state_true([(self.get_method, self.response, 200)])

    def test_preserve_list(self):
        self.response['flavors'].append(
            {
                "disk": 3,
                "id": cleanup_service.CONF_FLAVORS[0],
                "links": [{
                    "href": "http://openstack.ex.com/v2/openstack/flavors/3",
                    "rel": "self"}, {
                    "href": "http://openstack.ex.com/openstack/flavors/3",
                    "rel": "bookmark"}],
                "name": "m1.tiny",
                "ram": 512,
                "swap": 1,
                "vcpus": 1
            })
        self._test_is_preserve_true([(self.get_method, self.response, 200)])

    def test_prefix_opt_precedence(self):
        delete_mock = [(self.filter_saved_state, [], None),
                       (self.filter_resource_list, [], None),
                       (self.filter_prefix, [], None),
                       (self.get_method, self.response, 200),
                       (self.delete_method, 'error', None),
                       (self.log_method, 'exception', None)]
        self._test_prefix_opt_precedence(delete_mock)

    def test_resource_list_opt_precedence(self):
        delete_mock = [(self.filter_saved_state, [], None),
                       (self.filter_resource_list, [], None),
                       (self.filter_prefix, [], None),
                       (self.get_method, self.response, 200),
                       (self.delete_method, 'error', None),
                       (self.log_method, 'exception', None)]
        self._test_resource_list_opt_precedence(delete_mock)


class TestRoleService(BaseCmdServiceTests):

    service_class = 'RoleService'
    service_name = 'roles'
    response = {
        "roles": [
            {
                "domain_id": "FakeDomain",
                "id": "3efrt74r45hn",
                "name": "president",
                "links": {
                    "self": "http://ex.com/identity/v3/roles/3efrt74r45hn"
                }
            },
            {
                "domain_id": 'FakeDomain',
                "id": "39ruo5sdk040",
                "name": "vice-p",
                "links": {
                    "self": "http://ex.com/identity/v3/roles/39ruo5sdk040"
                }
            }
        ]
    }

    def test_delete_fail(self):
        delete_mock = [(self.get_method, self.response, 200),
                       (self.delete_method, 'error', None),
                       (self.log_method, 'exception', None)]
        self._test_delete(delete_mock, fail=True)

    def test_delete_pass(self):
        delete_mock = [(self.get_method, self.response, 200),
                       (self.delete_method, None, 204),
                       (self.log_method, 'exception', None)]
        self._test_delete(delete_mock)

    def test_dry_run(self):
        dry_mock = [(self.get_method, self.response, 200),
                    (self.delete_method, "delete", None)]
        self._test_dry_run_true(dry_mock)

    def test_save_state(self):
        self._test_saved_state_true([(self.get_method, self.response, 200)])

    def test_prefix_opt_precedence(self):
        delete_mock = [(self.filter_saved_state, [], None),
                       (self.filter_resource_list, [], None),
                       (self.filter_prefix, [], None),
                       (self.get_method, self.response, 200),
                       (self.delete_method, 'error', None),
                       (self.log_method, 'exception', None)]
        self._test_prefix_opt_precedence(delete_mock)

    def test_resource_list_opt_precedence(self):
        delete_mock = [(self.filter_saved_state, [], None),
                       (self.filter_resource_list, [], None),
                       (self.filter_prefix, [], None),
                       (self.get_method, self.response, 200),
                       (self.delete_method, 'error', None),
                       (self.log_method, 'exception', None)]
        self._test_resource_list_opt_precedence(delete_mock)


class TestUserService(BaseCmdServiceTests):

    service_class = 'UserService'
    service_name = 'users'
    response = {
        "users": [
            {
                "domain_id": "TempestDomain",
                "enabled": True,
                "id": "e812fb332456423fdv1b1320121qwe2",
                "links": {
                    "self": "http://example.com/identity/v3/users/"
                            "e812fb332456423fdv1b1320121qwe2",
                },
                "name": "Thunder",
                "password_expires_at": "3102-11-06T15:32:17.000000",
            },
            {
                "domain_id": "TempestDomain",
                "enabled": True,
                "id": "32rwef64245tgr20121qw324bgg",
                "links": {
                    "self": "http://example.com/identity/v3/users/"
                            "32rwef64245tgr20121qw324bgg",
                },
                "name": "Lightning",
                "password_expires_at": "1893-11-06T15:32:17.000000",
            }
        ]
    }

    def test_delete_fail(self):
        delete_mock = [(self.get_method, self.response, 200),
                       (self.delete_method, 'error', None),
                       (self.log_method, 'exception', None)]
        self._test_delete(delete_mock, fail=True)

    def test_delete_pass(self):
        delete_mock = [(self.get_method, self.response, 200),
                       (self.delete_method, None, 204),
                       (self.log_method, 'exception', None)]
        self._test_delete(delete_mock)

    def test_dry_run(self):
        dry_mock = [(self.get_method, self.response, 200),
                    (self.delete_method, "delete", None)]
        self._test_dry_run_true(dry_mock)

    def test_save_state(self):
        self._test_saved_state_true([(self.get_method, self.response, 200)])

    def test_preserve_list(self):
        self.response['users'].append(
            {
                "domain_id": "TempestDomain",
                "enabled": True,
                "id": "23ads5tg3rtrhe30121qwhyth",
                "links": {
                    "self": "http://example.com/identity/v3/users/"
                            "23ads5tg3rtrhe30121qwhyth",
                },
                "name": cleanup_service.CONF_USERS[0],
                "password_expires_at": "1893-11-06T15:32:17.000000",
            })
        self._test_is_preserve_true([(self.get_method, self.response, 200)])

    def test_prefix_opt_precedence(self):
        delete_mock = [(self.filter_saved_state, [], None),
                       (self.filter_resource_list, [], None),
                       (self.filter_prefix, [], None),
                       (self.get_method, self.response, 200),
                       (self.delete_method, 'error', None),
                       (self.log_method, 'exception', None)]
        self._test_prefix_opt_precedence(delete_mock)

    def test_resource_list_opt_precedence(self):
        delete_mock = [(self.filter_saved_state, [], None),
                       (self.filter_resource_list, [], None),
                       (self.filter_prefix, [], None),
                       (self.get_method, self.response, 200),
                       (self.delete_method, 'error', None),
                       (self.log_method, 'exception', None)]
        self._test_resource_list_opt_precedence(delete_mock)



# Copyright 2015 NEC Corporation.  All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from oslo_config import cfg
import testtools

from tempest.api.compute import base as compute_base
from tempest import config
from tempest.lib import exceptions
from tempest.tests import base
from tempest.tests import fake_config


class VersionTestNoneTolatest(compute_base.BaseV2ComputeTest):
    min_microversion = None
    max_microversion = 'latest'


class VersionTestNoneTo2_2(compute_base.BaseV2ComputeTest):
    min_microversion = None
    max_microversion = '2.2'


class VersionTest2_3ToLatest(compute_base.BaseV2ComputeTest):
    min_microversion = '2.3'
    max_microversion = 'latest'


class VersionTest2_5To2_10(compute_base.BaseV2ComputeTest):
    min_microversion = '2.5'
    max_microversion = '2.10'


class VersionTest2_10To2_10(compute_base.BaseV2ComputeTest):
    min_microversion = '2.10'
    max_microversion = '2.10'


class InvalidVersionTest(compute_base.BaseV2ComputeTest):
    min_microversion = '2.11'
    max_microversion = '2.1'


class TestMicroversionsTestsClass(base.TestCase):

    def setUp(self):
        super(TestMicroversionsTestsClass, self).setUp()
        self.useFixture(fake_config.ConfigFixture())
        self.patchobject(config, 'TempestConfigPrivate',
                         fake_config.FakePrivate)

    def _test_version(self, cfg_min, cfg_max,
                      expected_pass_tests,
                      expected_skip_tests):
        cfg.CONF.set_default('min_microversion',
                             cfg_min, group='compute')
        cfg.CONF.set_default('max_microversion',
                             cfg_max, group='compute')
        try:
            for test_class in expected_pass_tests:
                test_class.skip_checks()
            for test_class in expected_skip_tests:
                self.assertRaises(testtools.TestCase.skipException,
                                  test_class.skip_checks)
        except testtools.TestCase.skipException as e:
            raise testtools.TestCase.failureException(str(e))

    def test_config_version_none_none(self):
        expected_pass_tests = [VersionTestNoneTolatest, VersionTestNoneTo2_2]
        expected_skip_tests = [VersionTest2_3ToLatest, VersionTest2_5To2_10,
                               VersionTest2_10To2_10]
        self._test_version(None, None,
                           expected_pass_tests,
                           expected_skip_tests)

    def test_config_version_none_23(self):
        expected_pass_tests = [VersionTestNoneTolatest, VersionTestNoneTo2_2,
                               VersionTest2_3ToLatest]
        expected_skip_tests = [VersionTest2_5To2_10, VersionTest2_10To2_10]
        self._test_version(None, '2.3',
                           expected_pass_tests,
                           expected_skip_tests)

    def test_config_version_22_latest(self):
        expected_pass_tests = [VersionTestNoneTolatest, VersionTestNoneTo2_2,
                               VersionTest2_3ToLatest, VersionTest2_5To2_10,
                               VersionTest2_10To2_10]
        expected_skip_tests = []
        self._test_version('2.2', 'latest',
                           expected_pass_tests,
                           expected_skip_tests)

    def test_config_version_22_23(self):
        expected_pass_tests = [VersionTestNoneTolatest, VersionTestNoneTo2_2,
                               VersionTest2_3ToLatest]
        expected_skip_tests = [VersionTest2_5To2_10, VersionTest2_10To2_10]
        self._test_version('2.2', '2.3',
                           expected_pass_tests,
                           expected_skip_tests)

    def test_config_version_210_210(self):
        expected_pass_tests = [VersionTestNoneTolatest,
                               VersionTest2_3ToLatest,
                               VersionTest2_5To2_10,
                               VersionTest2_10To2_10]
        expected_skip_tests = [VersionTestNoneTo2_2]
        self._test_version('2.10', '2.10',
                           expected_pass_tests,
                           expected_skip_tests)

    def test_config_version_none_latest(self):
        expected_pass_tests = [VersionTestNoneTolatest, VersionTestNoneTo2_2,
                               VersionTest2_3ToLatest, VersionTest2_5To2_10,
                               VersionTest2_10To2_10]
        expected_skip_tests = []
        self._test_version(None, 'latest',
                           expected_pass_tests,
                           expected_skip_tests)

    def test_config_version_latest_latest(self):
        expected_pass_tests = [VersionTestNoneTolatest, VersionTest2_3ToLatest]
        expected_skip_tests = [VersionTestNoneTo2_2, VersionTest2_5To2_10,
                               VersionTest2_10To2_10]
        self._test_version('latest', 'latest',
                           expected_pass_tests,
                           expected_skip_tests)

    def test_config_invalid_version(self):
        cfg.CONF.set_default('min_microversion',
                             '2.5', group='compute')
        cfg.CONF.set_default('max_microversion',
                             '2.1', group='compute')
        self.assertRaises(exceptions.InvalidAPIVersionRange,
                          VersionTestNoneTolatest.skip_checks)

    def test_config_version_invalid_test_version(self):
        cfg.CONF.set_default('min_microversion',
                             None, group='compute')
        cfg.CONF.set_default('max_microversion',
                             '2.13', group='compute')
        self.assertRaises(exceptions.InvalidAPIVersionRange,
                          InvalidVersionTest.skip_checks)



# Copyright 2022 NEC Corporation.  All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.api.compute import base
from tempest.lib import decorators


class ServerExternalEventsTest(base.BaseV2ComputeAdminTest):
    """Test server external events test"""

    # TODO(gmann): Remove the admin access to service user
    # once nova change the default of this API to service
    # role. To merge the nova changing the policy default
    # we need to use token with admin as well as service
    # role and later we can use only service token.
    credentials = ['primary', 'admin', ['service_user', 'admin', 'service']]

    @decorators.idempotent_id('6bbf4723-61d2-4372-af55-7ba27f1c9ba6')
    def test_create_server_external_events(self):
        """Test create a server and add some external events"""
        server_id = self.create_test_server(wait_until='ACTIVE')['id']
        events = [
            {
                "name": "network-changed",
                "server_uuid": server_id,
            }
        ]
        client = self.os_service_user.server_external_events_client
        events_resp = client.create_server_external_events(
            events=events)['events'][0]
        self.assertEqual(server_id, events_resp['server_uuid'])
        self.assertEqual('network-changed', events_resp['name'])
        self.assertEqual(200, events_resp['code'])



# Copyright 2014 Dell Inc.
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the
# License for the specific language governing permissions and limitations
# under the License.

"""
Utility for cleaning up environment after Tempest test run

**Usage:** ``tempest cleanup [--help] [OPTIONS]``

If run with no arguments, ``tempest cleanup`` will query your OpenStack
deployment and build a list of resources to delete and destroy them. This list
will exclude the resources from ``saved_state.json`` and will include the
configured admin account if the ``--delete-tempest-conf-objects`` flag is
specified. By default the admin project is not deleted and the admin user
specified in ``tempest.conf`` is never deleted.

Example Run
-----------

.. warning::

    We advice not to run tempest cleanup on production environments.

.. warning::

    If step 1 is skipped in the example below, the cleanup procedure
    may delete resources that existed in the cloud before the test run. This
    may cause an unwanted destruction of cloud resources, so use caution with
    this command.

    Examples::

     $ tempest cleanup --init-saved-state
     $ # Actual running of Tempest tests
     $ tempest cleanup

Runtime Arguments
-----------------

* ``--init-saved-state``: Initializes the saved state of the OpenStack
  deployment and will output a ``saved_state.json`` file containing resources
  from your deployment that will be preserved from the cleanup command. This
  should be done prior to running Tempest tests. Note, that if other users of
  your cloud could have created resources after running ``--init-saved-state``,
  it would not protect those resources as they wouldn't be present in the
  saved_state.json file.

* ``--delete-tempest-conf-objects``: If option is present, then the command
  will delete the admin project in addition to the resources associated with
  them on clean up. If option is not present, the command will delete the
  resources associated with the Tempest and alternate Tempest users and
  projects but will not delete the projects themselves.

* ``--dry-run``: Creates a report (``./dry_run.json``) of the projects that
  will be cleaned up (in the ``_projects_to_clean`` dictionary [1]_) and the
  global objects that will be removed (domains, flavors, images, roles,
  projects, and users). Once the cleanup command is executed (e.g. run without
  parameters), running it again with ``--dry-run`` should yield an empty
  report. We STRONGLY ENCOURAGE to run ``tempest cleanup`` with ``--dry-run``
  first and then verify that the resources listed in the ``dry_run.json`` file
  are meant to be deleted.

* ``--prefix``: Only resources that match the prefix will be deleted. When this
  option is used, ``saved_state.json`` file is not needed (no need to run with
  ``--init-saved-state`` first).

  All tempest resources are created with the prefix value from the config
  option ``resource_name_prefix`` in tempest.conf. To cleanup only the
  resources created by tempest, you should use the prefix set in your
  tempest.conf (the default value of ``resource_name_prefix`` is ``tempest``.

  Note, that some resources are not named thus they will not be deleted when
  filtering based on the prefix. This option will be ignored when
  ``--init-saved-state`` is used so that it can capture the true init state -
  all resources present at that moment. If there is any ``saved_state.json``
  file present (e.g. if you ran the tempest cleanup with ``--init-saved-state``
  before) and you run the tempest cleanup with ``--prefix``, the
  ``saved_state.json`` file will be ignored and cleanup will be done based on
  the passed prefix only.

* ``--resource-list``: Allows the use of file ``./resource_list.json``, which
  contains all resources created by Tempest during all Tempest runs, to
  create another method for removing only resources created by Tempest.
  List of these resources is created when config option ``record_resources``
  in default section is set to true. After using this option for cleanup,
  the existing ``./resource_list.json`` is cleared from deleted resources.

  When this option is used, ``saved_state.json`` file is not needed (no
  need to run with ``--init-saved-state`` first). If there is any
  ``saved_state.json`` file present and you run the tempest cleanup with
  ``--resource-list``, the ``saved_state.json`` file will be ignored and
  cleanup will be done based on the ``resource_list.json`` only.

  If you run tempest cleanup with both ``--prefix`` and ``--resource-list``,
  the ``--resource-list`` option will be ignored and cleanup will be done
  based on the ``--prefix`` option only.

* ``--help``: Print the help text for the command and parameters.

.. [1] The ``_projects_to_clean`` dictionary in ``dry_run.json`` lists the
    projects that ``tempest cleanup`` will loop through to delete child
    objects, but the command will, by default, not delete the projects
    themselves. This may differ from the ``projects`` list as you can clean
    the Tempest and alternate Tempest users and projects but they will not be
    deleted unless the ``--delete-tempest-conf-objects`` flag is used to
    force their deletion.

.. note::

    If during execution of ``tempest cleanup`` NotImplemented exception
    occurres, ``tempest cleanup`` won't fail on that, it will be logged only.
    NotImplemented errors are ignored because they are an outcome of some
    extensions being disabled and ``tempest cleanup`` is not checking their
    availability as it tries to clean up as much as possible without any
    complicated logic.

"""
import sys
import traceback

from cliff import command
from oslo_log import log as logging
from oslo_serialization import jsonutils as json

from tempest import clients
from tempest.cmd import cleanup_service
from tempest.common import credentials_factory as credentials
from tempest import config
from tempest.lib import exceptions

SAVED_STATE_JSON = "saved_state.json"
DRY_RUN_JSON = "dry_run.json"
RESOURCE_LIST_JSON = "resource_list.json"
LOG = logging.getLogger(__name__)
CONF = config.CONF


class TempestCleanup(command.Command):

    GOT_EXCEPTIONS = []

    def take_action(self, parsed_args):
        try:
            self.init(parsed_args)
            if not parsed_args.init_saved_state:
                self._cleanup()
        except Exception:
            LOG.exception("Failure during cleanup")
            traceback.print_exc()
            raise
        # ignore NotImplemented errors as those are an outcome of some
        # extensions being disabled and cleanup is not checking their
        # availability as it tries to clean up as much as possible without
        # any complicated logic
        critical_exceptions = [ex for ex in self.GOT_EXCEPTIONS if
                               not isinstance(ex, exceptions.NotImplemented)]
        if critical_exceptions:
            raise Exception(self.GOT_EXCEPTIONS)

    def init(self, parsed_args):
        # set new handler for logging to stdout, by default only INFO messages
        # are logged to stdout
        stdout_handler = logging.logging.StreamHandler()
        # debug argument is defined in cliff already
        if self.app_args.debug:
            stdout_handler.level = logging.DEBUG
        else:
            stdout_handler.level = logging.INFO
        LOG.handlers.append(stdout_handler)

        cleanup_service.init_conf()
        self.options = parsed_args
        self.admin_mgr = clients.Manager(
            credentials.get_configured_admin_credentials())
        self.dry_run_data = {}
        self.resource_data = {}
        self.json_data = {}

        # available services
        self.project_associated_services = (
            cleanup_service.get_project_associated_cleanup_services())
        self.resource_cleanup_services = (
            cleanup_service.get_resource_cleanup_services())
        self.global_services = cleanup_service.get_global_cleanup_services()

        if parsed_args.init_saved_state:
            self._init_state()
            return

        if parsed_args.prefix:
            return

        if parsed_args.resource_list:
            self._load_resource_list()
            return

        self._load_saved_state()

    def _cleanup(self):
        LOG.info("Begin cleanup")
        is_dry_run = self.options.dry_run
        is_preserve = not self.options.delete_tempest_conf_objects
        is_resource_list = self.options.resource_list
        is_save_state = False
        cleanup_prefix = self.options.prefix

        if is_dry_run:
            self.dry_run_data["_projects_to_clean"] = {}

        admin_mgr = self.admin_mgr
        # Always cleanup tempest and alt tempest projects unless
        # they are in saved state json. Therefore is_preserve is False
        kwargs = {'data': self.dry_run_data,
                  'is_dry_run': is_dry_run,
                  'resource_list_json': self.resource_data,
                  'saved_state_json': self.json_data,
                  'is_preserve': False,
                  'is_resource_list': is_resource_list,
                  'is_save_state': is_save_state,
                  'prefix': cleanup_prefix}
        project_service = cleanup_service.ProjectService(admin_mgr, **kwargs)
        projects = project_service.list()
        LOG.info("Processing %s projects", len(projects))

        # Loop through list of projects and clean them up.
        for project in projects:
            self._clean_project(project)

        kwargs = {'data': self.dry_run_data,
                  'is_dry_run': is_dry_run,
                  'resource_list_json': self.resource_data,
                  'saved_state_json': self.json_data,
                  'is_preserve': is_preserve,
                  'is_resource_list': is_resource_list,
                  'is_save_state': is_save_state,
                  'prefix': cleanup_prefix,
                  'got_exceptions': self.GOT_EXCEPTIONS}
        LOG.info("Processing global services")
        for service in self.global_services:
            svc = service(admin_mgr, **kwargs)
            svc.run()

        LOG.info("Processing services")
        for service in self.resource_cleanup_services:
            svc = service(self.admin_mgr, **kwargs)
            svc.run()

        if is_dry_run:
            with open(DRY_RUN_JSON, 'w+') as f:
                f.write(json.dumps(self.dry_run_data, sort_keys=True,
                                   indent=2, separators=(',', ': ')))

        if is_resource_list:
            LOG.info("Clearing 'resource_list.json' file.")
            with open(RESOURCE_LIST_JSON, 'w') as f:
                f.write('{}')

    def _clean_project(self, project):
        LOG.debug("Cleaning project:  %s ", project['name'])
        is_dry_run = self.options.dry_run
        dry_run_data = self.dry_run_data
        is_preserve = not self.options.delete_tempest_conf_objects
        is_resource_list = self.options.resource_list
        project_id = project['id']
        project_name = project['name']
        project_data = None
        cleanup_prefix = self.options.prefix
        if is_dry_run:
            project_data = dry_run_data["_projects_to_clean"][project_id] = {}
            project_data['name'] = project_name

        kwargs = {'data': project_data,
                  'is_dry_run': is_dry_run,
                  'saved_state_json': self.json_data,
                  'resource_list_json': self.resource_data,
                  'is_preserve': is_preserve,
                  'is_resource_list': is_resource_list,
                  'is_save_state': False,
                  'project_id': project_id,
                  'prefix': cleanup_prefix,
                  'got_exceptions': self.GOT_EXCEPTIONS}
        for service in self.project_associated_services:
            svc = service(self.admin_mgr, **kwargs)
            svc.run()

    def get_parser(self, prog_name):
        parser = super(TempestCleanup, self).get_parser(prog_name)
        parser.add_argument('--init-saved-state', action="store_true",
                            dest='init_saved_state', default=False,
                            help="Creates JSON file: " + SAVED_STATE_JSON +
                            ", representing the current state of your "
                            "deployment,  specifically object types "
                            "tempest creates and destroys during a run. "
                            "You must run with this flag prior to "
                            "executing cleanup in normal mode, which is with "
                            "no arguments.")
        parser.add_argument('--delete-tempest-conf-objects',
                            action="store_true",
                            dest='delete_tempest_conf_objects',
                            default=False,
                            help="Force deletion of the tempest and "
                            "alternate tempest users and projects.")
        parser.add_argument('--dry-run', action="store_true",
                            dest='dry_run', default=False,
                            help="Generate JSON file:" + DRY_RUN_JSON +
                            ", that reports the objects that would have "
                            "been deleted had a full cleanup been run.")
        parser.add_argument('--prefix', dest='prefix', default=None,
                            help="Only resources that match the prefix will "
                            "be deleted (resources in saved_state.json are "
                            "not taken into account). All tempest resources "
                            "are created with the prefix value set by "
                            "resource_name_prefix in tempest.conf, default "
                            "prefix is tempest. Note that some resources are "
                            "not named thus they will not be deleted when "
                            "filtering based on the prefix. This opt will be "
                            "ignored when --init-saved-state is used so that "
                            "it can capture the true init state - all "
                            "resources present at that moment.")
        parser.add_argument('--resource-list', action="store_true",
                            dest='resource_list', default=False,
                            help="Runs tempest cleanup with generated "
                            "JSON file: " + RESOURCE_LIST_JSON + " to "
                            "erase resources created during Tempest run. "
                            "NOTE: To create " + RESOURCE_LIST_JSON + " "
                            "set config option record_resources under default "
                            "section in tempest.conf file to true. This "
                            "option will be ignored when --init-saved-state "
                            "is used so that it can capture the true init "
                            "state - all resources present at that moment. "
                            "This option will be ignored if passed with "
                            "--prefix.")
        return parser

    def get_description(self):
        return ('tempest cleanup tool, read the full documentation before '
                'using this tool. We advice not to run it on production '
                'environments. On environments where also other users may '
                'create resources, we strongly advice using --dry-run '
                'argument first and verify the content of dry_run.json file.')

    def _init_state(self):
        LOG.info("Initializing saved state.")
        data = {}
        admin_mgr = self.admin_mgr
        kwargs = {'data': data,
                  'is_dry_run': False,
                  'saved_state_json': data,
                  'is_preserve': False,
                  'is_resource_list': False,
                  'is_save_state': True,
                  # must be None as we want to capture true init state
                  # (all resources present) thus no filtering based
                  # on the prefix
                  'prefix': None,
                  'got_exceptions': self.GOT_EXCEPTIONS}
        for service in self.global_services:
            svc = service(admin_mgr, **kwargs)
            svc.run()

        for service in self.project_associated_services:
            svc = service(admin_mgr, **kwargs)
            svc.run()

        for service in self.resource_cleanup_services:
            svc = service(admin_mgr, **kwargs)
            svc.run()

        with open(SAVED_STATE_JSON, 'w+') as f:
            f.write(json.dumps(data, sort_keys=True,
                               indent=2, separators=(',', ': ')))

    def _load_resource_list(self, resource_list_json=RESOURCE_LIST_JSON):
        try:
            with open(resource_list_json, 'rb') as json_file:
                self.resource_data = json.load(json_file)
        except IOError as ex:
            LOG.exception(
                "Failed loading 'resource_list.json', please "
                "be sure you created this file by setting config "
                "option record_resources in default section to true "
                "prior to running tempest. Exception: %s", ex)
            sys.exit(ex)
        except Exception as ex:
            LOG.exception(
                "Exception parsing 'resource_list.json' : %s", ex)
            sys.exit(ex)

    def _load_saved_state(self, saved_state_json=SAVED_STATE_JSON):
        try:
            with open(saved_state_json, 'rb') as json_file:
                self.json_data = json.load(json_file)
        except IOError as ex:
            LOG.exception(
                "Failed loading saved state, please be sure you"
                " have first run cleanup with --init-saved-state "
                "flag prior to running tempest. Exception: %s", ex)
            sys.exit(ex)
        except Exception as ex:
            LOG.exception("Exception parsing saved state json : %s", ex)
            sys.exit(ex)



# Copyright 2016-2017 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import struct
import urllib.parse as urlparse
import urllib3

from tempest.api.compute import base
from tempest.common import compute
from tempest import config
from tempest.lib import decorators

CONF = config.CONF


class NoVNCConsoleTestJSON(base.BaseV2ComputeTest):
    """Test novnc console"""

    create_default_network = True

    @classmethod
    def skip_checks(cls):
        super(NoVNCConsoleTestJSON, cls).skip_checks()
        if not CONF.compute_feature_enabled.vnc_console:
            raise cls.skipException('VNC Console feature is disabled.')

    def setUp(self):
        super(NoVNCConsoleTestJSON, self).setUp()
        self._websocket = None

    def tearDown(self):
        super(NoVNCConsoleTestJSON, self).tearDown()
        if self._websocket is not None:
            self._websocket.close()
        # NOTE(zhufl): Because server_check_teardown will raise Exception
        # which will prevent other cleanup steps from being executed, so
        # server_check_teardown should be called after super's tearDown.
        self.server_check_teardown()

    @classmethod
    def setup_clients(cls):
        super(NoVNCConsoleTestJSON, cls).setup_clients()
        cls.client = cls.servers_client

    @classmethod
    def resource_setup(cls):
        super(NoVNCConsoleTestJSON, cls).resource_setup()
        cls.server = cls.create_test_server(wait_until="ACTIVE")
        cls.use_get_remote_console = False
        if not cls.is_requested_microversion_compatible('2.5'):
            cls.use_get_remote_console = True

    def _validate_novnc_html(self, vnc_url):
        """Verify we can connect to novnc and get back the javascript."""
        resp = urllib3.PoolManager().request('GET', vnc_url)
        # Make sure that the GET request was accepted by the novncproxy
        self.assertEqual(resp.status, 200, 'Got a Bad HTTP Response on the '
                         'initial call: ' + str(resp.status))
        # Do some basic validation to make sure it is an expected HTML document
        resp_data = resp.data.decode()
        # This is needed in the case of example: <html lang="en">
        self.assertRegex(resp_data, '<html.*>',
                         'Not a valid html document in the response.')
        self.assertIn('</html>', resp_data,
                      'Not a valid html document in the response.')
        # Just try to make sure we got JavaScript back for noVNC, since we
        # won't actually use it since not inside of a browser
        self.assertIn('noVNC', resp_data,
                      'Not a valid noVNC javascript html document.')
        self.assertIn('<script', resp_data,
                      'Not a valid noVNC javascript html document.')

    def _validate_rfb_negotiation(self):
        """Verify we can connect to novnc and do the websocket connection."""
        # Turn the Socket into a WebSocket to do the communication
        data = self._websocket.receive_frame()
        self.assertFalse(data is None or not data,
                         'Token must be invalid because the connection '
                         'closed.')
        # Parse the RFB version from the data to make sure it is valid
        # and belong to the known supported RFB versions.
        version = float("%d.%d" % (int(data[4:7], base=10),
                                   int(data[8:11], base=10)))
        # Add the max RFB versions supported
        supported_versions = [3.3, 3.8]
        self.assertIn(version, supported_versions,
                      'Bad RFB Version: ' + str(version))
        # Send our RFB version to the server
        self._websocket.send_frame(data)
        # Get the sever authentication type and make sure None is supported
        data = self._websocket.receive_frame()
        self.assertIsNotNone(data, 'Expected authentication type None.')
        data_length = len(data)
        if version == 3.3:
            # For RFB 3.3: in the security handshake, rather than a two-way
            # negotiation, the server decides the security type and sends a
            # single word(4 bytes).
            self.assertEqual(
                data_length, 4, 'Expected authentication type None.')
            self.assertIn(1, [int(data[i]) for i in (0, 3)],
                          'Expected authentication type None.')
        else:
            self.assertGreaterEqual(
                len(data), 2, 'Expected authentication type None.')
            self.assertIn(
                1,
                [int(data[i + 1]) for i in range(int(data[0]))],
                'Expected authentication type None.')
            # Send to the server that we only support authentication
            # type None
            self._websocket.send_frame(bytes((1,)))

            # The server should send 4 bytes of 0's if security
            # handshake succeeded
            data = self._websocket.receive_frame()
            self.assertEqual(
                len(data), 4,
                'Server did not think security was successful.')
            self.assertEqual(
                [int(i) for i in data], [0, 0, 0, 0],
                'Server did not think security was successful.')

        # Say to leave the desktop as shared as part of client initialization
        self._websocket.send_frame(bytes((1,)))
        # Get the server initialization packet back and make sure it is the
        # right structure where bytes 20-24 is the name length and
        # 24-N is the name
        data = self._websocket.receive_frame()
        data_length = len(data) if data is not None else 0
        self.assertFalse(data_length <= 24 or
                         data_length != (struct.unpack(">L",
                                                       data[20:24])[0] + 24),
                         'Server initialization was not the right format.')
        # Since the rest of the data on the screen is arbitrary, we will
        # close the socket and end our validation of the data at this point
        # Assert that the latest check was false, meaning that the server
        # initialization was the right format
        self.assertFalse(data_length <= 24 or
                         data_length != (struct.unpack(">L",
                                                       data[20:24])[0] + 24))

    def _validate_websocket_upgrade(self):
        """Verify that the websocket upgrade was successful.

        Parses response and ensures that required response
        fields are present and accurate.
        (https://tools.ietf.org/html/rfc7231#section-6.2.2)
        """

        self.assertTrue(
            self._websocket.response.startswith(b'HTTP/1.1 101 Switching '
                                                b'Protocols'),
            'Incorrect HTTP return status code: {}'.format(
                str(self._websocket.response)
            )
        )
        _required_header = 'upgrade: websocket'
        _response = str(self._websocket.response).lower()
        self.assertIn(
            _required_header,
            _response,
            'Did not get the expected WebSocket HTTP Response.'
        )

    @decorators.idempotent_id('c640fdff-8ab4-45a4-a5d8-7e6146cbd0dc')
    def test_novnc(self):
        """Test accessing novnc console of server"""
        if self.use_get_remote_console:
            body = self.client.get_remote_console(
                self.server['id'], console_type='novnc',
                protocol='vnc')['remote_console']
        else:
            body = self.client.get_vnc_console(self.server['id'],
                                               type='novnc')['console']
        self.assertEqual('novnc', body['type'])
        # Do the initial HTTP Request to novncproxy to get the NoVNC JavaScript
        self._validate_novnc_html(body['url'])
        # Do the WebSockify HTTP Request to novncproxy to do the RFB connection
        self._websocket = compute.create_websocket(body['url'])
        # Validate that we successfully connected and upgraded to Web Sockets
        self._validate_websocket_upgrade()
        # Validate the RFB Negotiation to determine if a valid VNC session
        self._validate_rfb_negotiation()

    @decorators.idempotent_id('f9c79937-addc-4aaa-9e0e-841eef02aeb7')
    def test_novnc_bad_token(self):
        """Test accessing novnc console with bad token

        Do the WebSockify HTTP Request to novnc proxy with a bad token,
        the novnc proxy should reject the connection and closed it.
        """
        if self.use_get_remote_console:
            body = self.client.get_remote_console(
                self.server['id'], console_type='novnc',
                protocol='vnc')['remote_console']
        else:
            body = self.client.get_vnc_console(self.server['id'],
                                               type='novnc')['console']
        self.assertEqual('novnc', body['type'])
        # Do the WebSockify HTTP Request to novncproxy with a bad token
        parts = urlparse.urlparse(body['url'])
        qparams = urlparse.parse_qs(parts.query)
        if 'path' in qparams:
            qparams['path'] = urlparse.unquote(qparams['path'][0]).replace(
                'token=', 'token=bad')
        elif 'token' in qparams:
            qparams['token'] = 'bad' + qparams['token'][0]
        new_query = urlparse.urlencode(qparams)
        new_parts = urlparse.ParseResult(parts.scheme, parts.netloc,
                                         parts.path, parts.params, new_query,
                                         parts.fragment)
        url = urlparse.urlunparse(new_parts)
        self._websocket = compute.create_websocket(url)
        # Make sure the novncproxy rejected the connection and closed it
        data = self._websocket.receive_frame()
        self.assertTrue(data is None or not data,
                        "The novnc proxy actually sent us some data, but we "
                        "expected it to close the connection.")



# All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from tempest.api.image import base
from tempest import config
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc

CONF = config.CONF


class ImagesTagsNegativeTest(base.BaseV2ImageTest):
    """Negative tests of image tags"""

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('8cd30f82-6f9a-4c6e-8034-c1b51fba43d9')
    def test_update_tags_for_non_existing_image(self):
        """Update image tag with non existing image"""
        tag = data_utils.rand_name(
            prefix=CONF.resource_name_prefix, name='tag')
        non_exist_image = data_utils.rand_uuid()
        self.assertRaises(lib_exc.NotFound, self.client.add_image_tag,
                          non_exist_image, tag)

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('39c023a2-325a-433a-9eea-649bf1414b19')
    def test_delete_non_existing_tag(self):
        """Delete non existing image tag"""
        image = self.create_image(container_format='bare',
                                  disk_format='raw',
                                  visibility='private'
                                  )
        tag = data_utils.rand_name(
            prefix=CONF.resource_name_prefix, name='non-exist-tag')
        self.addCleanup(self.client.delete_image, image['id'])
        self.assertRaises(lib_exc.NotFound, self.client.delete_image_tag,
                          image['id'], tag)



# Copyright 2016 NEC Corporation.  All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.api.compute import base
from tempest.common import utils
from tempest import config
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc

CONF = config.CONF


class VolumesAdminNegativeTest(base.BaseV2ComputeAdminTest):
    """Negative tests of volume swapping"""

    create_default_network = True

    @classmethod
    def setup_credentials(cls):
        cls.prepare_instance_network()
        super(VolumesAdminNegativeTest, cls).setup_credentials()

    @classmethod
    def skip_checks(cls):
        super(VolumesAdminNegativeTest, cls).skip_checks()
        if not CONF.service_available.cinder:
            skip_msg = ("%s skipped as Cinder is not available" % cls.__name__)
            raise cls.skipException(skip_msg)

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('309b5ecd-0585-4a7e-a36f-d2b2bf55259d')
    def test_update_attached_volume_with_nonexistent_volume_in_uri(self):
        """Test swapping non existent volume should fail"""
        self.server = self.create_test_server(wait_until="ACTIVE")
        volume = self.create_volume()
        nonexistent_volume = data_utils.rand_uuid()
        self.assertRaises(lib_exc.NotFound,
                          self.admin_servers_client.update_attached_volume,
                          self.server['id'], nonexistent_volume,
                          volumeId=volume['id'])

    @decorators.related_bug('1629110', status_code=400)
    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('7dcac15a-b107-46d3-a5f6-cb863f4e454a')
    def test_update_attached_volume_with_nonexistent_volume_in_body(self):
        """Test swapping volume to a non existence volume should fail"""
        validation_resources = self.get_class_validation_resources(
            self.os_primary)
        # NOTE(gibi): We need to wait for the guest to fully boot as
        # test_update_attached_volume_with_nonexistent_volume_in_body case
        # will attach a volume to it and therefore cleanup will try to detach
        # it. See bug 1960346 for details.
        self.server = self.create_test_server(
            validatable=True,
            validation_resources=validation_resources,
            wait_until="SSHABLE")

        volume = self.create_volume()
        self.attach_volume(self.server, volume)

        nonexistent_volume = data_utils.rand_uuid()
        self.assertRaises(lib_exc.BadRequest,
                          self.admin_servers_client.update_attached_volume,
                          self.server['id'], volume['id'],
                          volumeId=nonexistent_volume)


class UpdateMultiattachVolumeNegativeTest(base.BaseV2ComputeAdminTest):
    """Negative tests of swapping volume attached to multiple servers

    Negative tests of swapping volume attached to multiple servers with
    compute microversion greater than 2.59 and volume microversion greater
    than 3.26
    """

    min_microversion = '2.60'
    volume_min_microversion = '3.27'

    create_default_network = True

    @classmethod
    def setup_credentials(cls):
        cls.prepare_instance_network()
        super(UpdateMultiattachVolumeNegativeTest, cls).setup_credentials()

    @classmethod
    def skip_checks(cls):
        super(UpdateMultiattachVolumeNegativeTest, cls).skip_checks()
        if not CONF.compute_feature_enabled.volume_multiattach:
            raise cls.skipException('Volume multi-attach is not available.')

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('7576d497-b7c6-44bd-9cc5-c5b4e50fec71')
    @utils.services('volume')
    def test_multiattach_rw_volume_update_failure(self):
        """Test swapping volume attached to multi-servers with read-write mode

        1. Create two volumes "vol1" and "vol2"
        2. Create two instances "server1" and "server2"
        3. Attach "vol1" to both of these instances
        4. By default both of these attachments should have an attach_mode of
           read-write, so trying to swap "vol1" to "vol2" should fail
        5. Check "vol1" is still attached to both servers
        6. Check "vol2" is not attached to any server
        """
        multiattach_vol_type = CONF.volume.volume_type_multiattach

        # Create two multiattach capable volumes.
        vol1 = self.create_volume(volume_type=multiattach_vol_type)
        vol2 = self.create_volume(volume_type=multiattach_vol_type)

        # Create two instances.
        validation_resources = self.get_class_validation_resources(
            self.os_primary)
        # NOTE(gibi): We need to wait for the guests to fully boot as the test
        # will attach volumes to the servers and therefore cleanup will try to
        # detach them. See bug 1960346 for details.
        server1 = self.create_test_server(
            validatable=True,
            validation_resources=validation_resources,
            wait_until='SSHABLE'
        )
        server2 = self.create_test_server(
            validatable=True,
            validation_resources=validation_resources,
            wait_until='SSHABLE'
        )

        # Attach vol1 to both of these instances.
        vol1_attachment1 = self.attach_volume(server1, vol1)
        vol1_attachment2 = self.attach_volume(server2, vol1)

        # Assert that we now have two attachments.
        vol1 = self.volumes_client.show_volume(vol1['id'])['volume']
        self.assertEqual(2, len(vol1['attachments']))

        # By default both of these attachments should have an attach_mode of
        # read-write, assert that here to ensure the following calls to update
        # the volume will be rejected.
        for volume_attachment in vol1['attachments']:
            attachment_id = volume_attachment['attachment_id']
            attachment = self.attachments_client.show_attachment(
                attachment_id)['attachment']
            self.assertEqual('rw', attachment['attach_mode'])

        # Assert that a BadRequest is raised when we attempt to update volume1
        # to volume2 on server1 or server2.
        self.assertRaises(lib_exc.BadRequest,
                          self.admin_servers_client.update_attached_volume,
                          server1['id'], vol1['id'], volumeId=vol2['id'])
        self.assertRaises(lib_exc.BadRequest,
                          self.admin_servers_client.update_attached_volume,
                          server2['id'], vol1['id'], volumeId=vol2['id'])

        # Fetch the volume 1 to check the current attachments.
        vol1 = self.volumes_client.show_volume(vol1['id'])['volume']
        vol1_attachment_ids = [a['id'] for a in vol1['attachments']]

        # Assert that volume 1 is still attached to both server 1 and 2.
        self.assertIn(vol1_attachment1['id'], vol1_attachment_ids)
        self.assertIn(vol1_attachment2['id'], vol1_attachment_ids)

        # Assert that volume 2 has no attachments.
        vol2 = self.volumes_client.show_volume(vol2['id'])['volume']
        self.assertEqual([], vol2['attachments'])



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import copy

from oslo_log import log as logging

from tempest.lib.common.utils import test_utils
from tempest.lib import exceptions

LOG = logging.getLogger(__name__)


def get_network_from_name(name, compute_networks_client):
    """Get a full network dict from just a network name

    :param str name: the name of the network to use
    :param network.NetworksClient compute_networks_client: The network client
        object to use for making the network lists api request
    :return: The full dictionary for the network in question
    :rtype: dict
    :raises InvalidTestResource: If the name provided is invalid, the networks
        list returns a 404, there are no found networks, or the found network
        is invalid
    """
    caller = test_utils.find_test_caller()

    if not name:
        raise exceptions.InvalidTestResource(type='network', name=name)

    networks = compute_networks_client.list_networks()['networks']
    # NOTE(zhufl) compute networks_client uses 'label' as network name field,
    # while neutron networks_client uses 'name' as network name field.
    try:
        networks = [n for n in networks if n['label'] == name]
    except KeyError:
        networks = [n for n in networks if n['name'] == name]

    # Check that a network exists, else raise an InvalidConfigurationException
    if len(networks) == 1:
        network = sorted(networks)[0]
    elif len(networks) > 1:
        msg = ("Network with name: %s had multiple matching networks in the "
               "list response: %s\n Unable to specify a single network" % (
                   name, networks))
        if caller:
            msg = '(%s) %s' % (caller, msg)
        LOG.warning(msg)
        raise exceptions.InvalidTestResource(type='network', name=name)
    else:
        msg = "Network with name: %s not found" % name
        if caller:
            msg = '(%s) %s' % (caller, msg)
        LOG.warning(msg)
        raise exceptions.InvalidTestResource(type='network', name=name)
    # To be consistent between neutron and nova network always use name even
    # if label is used in the api response. If neither is present than then
    # the returned network is invalid.
    name = network.get('name') or network.get('label')
    if not name:
        msg = "Network found from list doesn't contain a valid name or label"
        if caller:
            msg = '(%s) %s' % (caller, msg)
        LOG.warning(msg)
        raise exceptions.InvalidTestResource(type='network', name=name)
    network['name'] = name
    return network


def get_tenant_network(creds_provider, compute_networks_client,
                       shared_network_name):
    """Get a network usable by the primary tenant

    :param creds_provider: instance of credential provider
    :param compute_networks_client: compute network client. We want to have the
           compute network client so we can have use a common approach for both
           neutron and nova-network cases. If this is not an admin network
           client, set_network_kwargs might fail in case fixed_network_name
           is the network to be used, and it's not visible to the tenant
    :param shared_network_name: name of the shared network to be used if no
           tenant network is available in the creds provider
    :returns: a dict with 'id' and 'name' of the network
    """
    caller = test_utils.find_test_caller()
    net_creds = creds_provider.get_primary_creds()
    network = getattr(net_creds, 'network', None)
    if not network or not network.get('name'):
        if shared_network_name:
            msg = ('No valid network provided or created, defaulting to '
                   'fixed_network_name')
            if caller:
                msg = '(%s) %s' % (caller, msg)
            LOG.debug(msg)
            try:
                network = get_network_from_name(shared_network_name,
                                                compute_networks_client)
            except exceptions.InvalidTestResource:
                network = {}
    msg = ('Found network %s available for tenant' % network)
    if caller:
        msg = '(%s) %s' % (caller, msg)
    LOG.info(msg)
    return network


def set_networks_kwarg(network, kwargs=None):
    """Set 'networks' kwargs for a server create if missing

    :param network: dict of network to be used with 'id' and 'name'
    :param kwargs: server create kwargs to be enhanced
    :return: new dict of kwargs updated to include networks
    """
    params = copy.copy(kwargs) or {}
    if kwargs and 'networks' in kwargs:
        return params

    if network:
        if 'id' in network.keys():
            params.update({"networks": [{'uuid': network['id']}]})
        else:
            LOG.warning('The provided network dict: %s was invalid and did '
                        'not contain an id', network)
    return params



# Copyright 2015 NEC Corporation.  All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.lib.services.compute import networks_client
from tempest.tests.lib import fake_auth_provider
from tempest.tests.lib.services import base


class TestNetworksClient(base.BaseServiceTest):

    FAKE_NETWORK = {
        "bridge": None,
        "vpn_public_port": None,
        "dhcp_start": None,
        "bridge_interface": None,
        "share_address": None,
        "updated_at": None,
        "id": "34d5ae1e-5659-49cf-af80-73bccd7d7ad3",
        "cidr_v6": None,
        "deleted_at": None,
        "gateway": None,
        "rxtx_base": None,
        "label": '30d7',
        "priority": None,
        "project_id": None,
        "vpn_private_address": None,
        "deleted": None,
        "vlan": None,
        "broadcast": None,
        "netmask": None,
        "injected": None,
        "cidr": None,
        "vpn_public_address": None,
        "multi_host": None,
        "enable_dhcp": None,
        "dns2": None,
        "created_at": None,
        "host": None,
        "mtu": None,
        "gateway_v6": None,
        "netmask_v6": None,
        "dhcp_server": None,
        "dns1": None
        }

    network_id = "34d5ae1e-5659-49cf-af80-73bccd7d7ad3"

    FAKE_NETWORKS = [FAKE_NETWORK]

    def setUp(self):
        super(TestNetworksClient, self).setUp()
        fake_auth = fake_auth_provider.FakeAuthProvider()
        self.client = networks_client.NetworksClient(
            fake_auth, 'compute', 'regionOne')

    def _test_list_networks(self, bytes_body=False):
        fake_list = {"networks": self.FAKE_NETWORKS}
        self.check_service_client_function(
            self.client.list_networks,
            'tempest.lib.common.rest_client.RestClient.get',
            fake_list,
            bytes_body)

    def test_list_networks_with_str_body(self):
        self._test_list_networks()

    def test_list_networks_with_bytes_body(self):
        self._test_list_networks(bytes_body=True)

    def _test_show_network(self, bytes_body=False):
        self.check_service_client_function(
            self.client.show_network,
            'tempest.lib.common.rest_client.RestClient.get',
            {"network": self.FAKE_NETWORK},
            bytes_body,
            network_id=self.network_id
            )

    def test_show_network_with_str_body(self):
        self._test_show_network()

    def test_show_network_with_bytes_body(self):
        self._test_show_network(bytes_body=True)



# Copyright 2016 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from urllib import parse as urllib

from oslo_serialization import jsonutils as json

from tempest.lib.common import rest_client


class RolesClient(rest_client.RestClient):
    api_version = "v3"

    def create_role(self, **kwargs):
        """Create a Role.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/identity/v3/index.html#create-role
        """
        post_body = json.dumps({'role': kwargs})
        resp, body = self.post('roles', post_body)
        self.expected_success(201, resp.status)
        body = json.loads(body)
        return rest_client.ResponseBody(resp, body)

    def show_role(self, role_id):
        """GET a Role."""
        resp, body = self.get('roles/%s' % role_id)
        self.expected_success(200, resp.status)
        body = json.loads(body)
        return rest_client.ResponseBody(resp, body)

    def list_roles(self, **params):
        """Get the list of Roles.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/identity/v3/index.html#list-roles
        """
        url = 'roles'
        if params:
            url += '?%s' % urllib.urlencode(params)
        resp, body = self.get(url)
        self.expected_success(200, resp.status)
        body = json.loads(body)
        return rest_client.ResponseBody(resp, body)

    def update_role(self, role_id, **kwargs):
        """Update a Role.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/identity/v3/index.html#update-role
        """
        post_body = json.dumps({'role': kwargs})
        resp, body = self.patch('roles/%s' % role_id, post_body)
        self.expected_success(200, resp.status)
        body = json.loads(body)
        return rest_client.ResponseBody(resp, body)

    def delete_role(self, role_id):
        """Delete a role."""
        resp, body = self.delete('roles/%s' % role_id)
        self.expected_success(204, resp.status)
        return rest_client.ResponseBody(resp, body)

    def create_user_role_on_project(self, project_id, user_id, role_id):
        """Add roles to a user on a project."""
        resp, body = self.put('projects/%s/users/%s/roles/%s' %
                              (project_id, user_id, role_id), None)
        self.expected_success(204, resp.status)
        return rest_client.ResponseBody(resp, body)

    def create_user_role_on_domain(self, domain_id, user_id, role_id):
        """Add roles to a user on a domain."""
        resp, body = self.put('domains/%s/users/%s/roles/%s' %
                              (domain_id, user_id, role_id), None)
        self.expected_success(204, resp.status)
        return rest_client.ResponseBody(resp, body)

    def create_user_role_on_system(self, user_id, role_id):
        """Add roles to a user on the system."""
        resp, body = self.put('system/users/%s/roles/%s' %
                              (user_id, role_id), None)
        self.expected_success(204, resp.status)
        return rest_client.ResponseBody(resp, body)

    def list_user_roles_on_project(self, project_id, user_id):
        """list roles of a user on a project."""
        resp, body = self.get('projects/%s/users/%s/roles' %
                              (project_id, user_id))
        self.expected_success(200, resp.status)
        body = json.loads(body)
        return rest_client.ResponseBody(resp, body)

    def list_user_roles_on_domain(self, domain_id, user_id):
        """list roles of a user on a domain."""
        resp, body = self.get('domains/%s/users/%s/roles' %
                              (domain_id, user_id))
        self.expected_success(200, resp.status)
        body = json.loads(body)
        return rest_client.ResponseBody(resp, body)

    def list_user_roles_on_system(self, user_id):
        """list roles of a user on the system."""
        resp, body = self.get('system/users/%s/roles' % user_id)
        self.expected_success(200, resp.status)
        body = json.loads(body)
        return rest_client.ResponseBody(resp, body)

    def delete_role_from_user_on_project(self, project_id, user_id, role_id):
        """Delete role of a user on a project."""
        resp, body = self.delete('projects/%s/users/%s/roles/%s' %
                                 (project_id, user_id, role_id))
        self.expected_success(204, resp.status)
        return rest_client.ResponseBody(resp, body)

    def delete_role_from_user_on_domain(self, domain_id, user_id, role_id):
        """Delete role of a user on a domain."""
        resp, body = self.delete('domains/%s/users/%s/roles/%s' %
                                 (domain_id, user_id, role_id))
        self.expected_success(204, resp.status)
        return rest_client.ResponseBody(resp, body)

    def delete_role_from_user_on_system(self, user_id, role_id):
        """Delete role of a user on the system."""
        resp, body = self.delete('system/users/%s/roles/%s' %
                                 (user_id, role_id))
        self.expected_success(204, resp.status)
        return rest_client.ResponseBody(resp, body)

    def check_user_role_existence_on_project(self, project_id,
                                             user_id, role_id):
        """Check role of a user on a project."""
        resp, _ = self.head('projects/%s/users/%s/roles/%s' %
                            (project_id, user_id, role_id))
        self.expected_success(204, resp.status)
        return rest_client.ResponseBody(resp)

    def check_user_role_existence_on_domain(self, domain_id,
                                            user_id, role_id):
        """Check role of a user on a domain."""
        resp, _ = self.head('domains/%s/users/%s/roles/%s' %
                            (domain_id, user_id, role_id))
        self.expected_success(204, resp.status)
        return rest_client.ResponseBody(resp)

    def check_user_role_existence_on_system(self, user_id, role_id):
        """Check role of a user on the system."""
        resp, body = self.head('system/users/%s/roles/%s' % (user_id, role_id))
        self.expected_success(204, resp.status)
        return rest_client.ResponseBody(resp)

    def create_group_role_on_project(self, project_id, group_id, role_id):
        """Add roles to a group on a project."""
        resp, body = self.put('projects/%s/groups/%s/roles/%s' %
                              (project_id, group_id, role_id), None)
        self.expected_success(204, resp.status)
        return rest_client.ResponseBody(resp, body)

    def create_group_role_on_domain(self, domain_id, group_id, role_id):
        """Add roles to a group on a domain."""
        resp, body = self.put('domains/%s/groups/%s/roles/%s' %
                              (domain_id, group_id, role_id), None)
        self.expected_success(204, resp.status)
        return rest_client.ResponseBody(resp, body)

    def create_group_role_on_system(self, group_id, role_id):
        """Add roles to a group on the system."""
        resp, body = self.put('system/groups/%s/roles/%s' %
                              (group_id, role_id), None)
        self.expected_success(204, resp.status)
        return rest_client.ResponseBody(resp, body)

    def list_group_roles_on_project(self, project_id, group_id):
        """list roles of a group on a project."""
        resp, body = self.get('projects/%s/groups/%s/roles' %
                              (project_id, group_id))
        self.expected_success(200, resp.status)
        body = json.loads(body)
        return rest_client.ResponseBody(resp, body)

    def list_group_roles_on_domain(self, domain_id, group_id):
        """list roles of a group on a domain."""
        resp, body = self.get('domains/%s/groups/%s/roles' %
                              (domain_id, group_id))
        self.expected_success(200, resp.status)
        body = json.loads(body)
        return rest_client.ResponseBody(resp, body)

    def list_group_roles_on_system(self, group_id):
        """list roles of a group on the system."""
        resp, body = self.get('system/groups/%s/roles' % group_id)
        self.expected_success(200, resp.status)
        body = json.loads(body)
        return rest_client.ResponseBody(resp, body)

    def delete_role_from_group_on_project(self, project_id, group_id, role_id):
        """Delete role of a group on a project."""
        resp, body = self.delete('projects/%s/groups/%s/roles/%s' %
                                 (project_id, group_id, role_id))
        self.expected_success(204, resp.status)
        return rest_client.ResponseBody(resp, body)

    def delete_role_from_group_on_domain(self, domain_id, group_id, role_id):
        """Delete role of a group on a domain."""
        resp, body = self.delete('domains/%s/groups/%s/roles/%s' %
                                 (domain_id, group_id, role_id))
        self.expected_success(204, resp.status)
        return rest_client.ResponseBody(resp, body)

    def delete_role_from_group_on_system(self, group_id, role_id):
        """Delete role of a group on the system."""
        resp, body = self.delete('system/groups/%s/roles/%s' %
                                 (group_id, role_id))
        self.expected_success(204, resp.status)
        return rest_client.ResponseBody(resp, body)

    def check_role_from_group_on_project_existence(self, project_id,
                                                   group_id, role_id):
        """Check role of a group on a project."""
        resp, _ = self.head('projects/%s/groups/%s/roles/%s' %
                            (project_id, group_id, role_id))
        self.expected_success(204, resp.status)
        return rest_client.ResponseBody(resp)

    def check_role_from_group_on_domain_existence(self, domain_id,
                                                  group_id, role_id):
        """Check role of a group on a domain."""
        resp, _ = self.head('domains/%s/groups/%s/roles/%s' %
                            (domain_id, group_id, role_id))
        self.expected_success(204, resp.status)
        return rest_client.ResponseBody(resp)

    def check_role_from_group_on_system_existence(self, group_id, role_id):
        """Check role of a group on the system."""
        resp, body = self.head('system/groups/%s/roles/%s' %
                               (group_id, role_id))
        self.expected_success(204, resp.status)
        return rest_client.ResponseBody(resp)

    def create_role_inference_rule(self, prior_role, implies_role):
        """Create a role inference rule."""
        resp, body = self.put('roles/%s/implies/%s' %
                              (prior_role, implies_role), None)
        self.expected_success(201, resp.status)
        body = json.loads(body)
        return rest_client.ResponseBody(resp, body)

    def show_role_inference_rule(self, prior_role, implies_role):
        """Get a role inference rule."""
        resp, body = self.get('roles/%s/implies/%s' %
                              (prior_role, implies_role))
        self.expected_success(200, resp.status)
        body = json.loads(body)
        return rest_client.ResponseBody(resp, body)

    def list_role_inferences_rules(self, prior_role):
        """List the inferences rules from a role."""
        resp, body = self.get('roles/%s/implies' % prior_role)
        self.expected_success(200, resp.status)
        body = json.loads(body)
        return rest_client.ResponseBody(resp, body)

    def list_all_role_inference_rules(self):
        """Lists all role inference rules.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/identity/v3/index.html#list-all-role-inference-rules
        """
        resp, body = self.get('role_inferences')
        self.expected_success(200, resp.status)
        body = json.loads(body)
        return rest_client.ResponseBody(resp, body)

    def check_role_inference_rule(self, prior_role, implies_role):
        """Check a role inference rule."""
        resp, _ = self.head('roles/%s/implies/%s' %
                            (prior_role, implies_role))
        self.expected_success(204, resp.status)
        return rest_client.ResponseBody(resp)

    def delete_role_inference_rule(self, prior_role, implies_role):
        """Delete a role inference rule."""
        resp, _ = self.delete('roles/%s/implies/%s' %
                              (prior_role, implies_role))
        self.expected_success(204, resp.status)
        return rest_client.ResponseBody(resp)
